#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <gtkdatabox.h>

#include "phoebe_allocations.h"
#include "phoebe_calculations.h"
#include "phoebe_error_handling.h"
#include "phoebe_global.h"
#include "phoebe_gui.h"
#include "phoebe_gui_accessories.h"
#include "phoebe_gui_callbacks.h"
#include "phoebe_gui_exception_handling.h"
#include "phoebe_gui_scripting.h"
#include "phoebe_gui_support.h"
#include "phoebe_keywords.h"
#include "phoebe_logging_service.h"
#include "phoebe_scripting.h"

void parse_startup_line (int argc, char *argv[])
	{
	/* This function parses the command line and looks for known switches; it   */
	/* doesn't really do anything with them, it just writes what it found to    */
	/* the PHOEBE_args variable that is supposed to be used for real actions.   */

	int i;

	for (i = 1; i < argc; i++)
		{
		if ( (strcmp (argv[i],  "-h"   ) == 0) ||
		     (strcmp (argv[i],  "-?"   ) == 0) ||
				 (strcmp (argv[i], "--help") == 0) )
			{
			printf ("\nPHOEBE %s command line arguments: [-hsv] [keyword_file]\n\n", PHOEBE_VERSION_NUMBER);
			printf ("  -h, --help, -?      ..  this help screen\n");
			printf ("  -s, --script        ..  execute PHOEBE script\n");
			printf ("  -v, --version       ..  display PHOEBE version and exit\n");
			printf ("\n");
			exit (0);
			}
		if ( (strcmp (argv[i],  "-v"      ) == 0) ||
				 (strcmp (argv[i], "--version") == 0) )
			{
			printf ("\nPHOEBE %s, %s by Andrej Prsa\n", PHOEBE_VERSION_NUMBER, PHOEBE_VERSION_DATE);
			printf ("  Send comments and/or requests to andrej.prsa@fmf.uni-lj.si\n\n");
			exit (0);
			}
		if ( (strcmp (argv[i],  "-s"     ) == 0) ||
				 (strcmp (argv[i], "--script") == 0) )
			{
			/* We cannot start the script from here because the widgets aren't ini- */
			/* tialized yet; thus we just initialize the proper switches:           */
			PHOEBE_args.SCRIPT_SWITCH = 1;
			
			/* If someone forgot to put the script name, we should warn him and     */
			/* exit:                                                                */
			if ( (i+1 == argc) || (argv[i+1][0] == '-') )
				{
				printf ("\nPHOEBE %s command line arguments: [-hsv] [keyword_file]\n\n", PHOEBE_VERSION_NUMBER);
				printf ("  -h, --help, -?      ..  this help screen\n");
				printf ("  -s, --script        ..  execute PHOEBE script\n");
				printf ("  -v, --version       ..  display PHOEBE version and exit\n");
				printf ("\n");
				printf ("** ERROR: There is no argument given to the '-s' switch.\n");
				exit (0);
				}

			/* Otherwise let's read in the script's name:                           */
			declare_string (&PHOEBE_args.SCRIPT_NAME, argv[i+1]);
			i++; i++;                            /* This will skip the script name. */
			if (i >= argc) break;   /* If this was the last switch, break the loop. */
			}
		if ( argv[i][0] != '-' )
			{
			/* This means that the command line argument doesn't contain '-'; thus  */
			/* it should be a keyword file. All other arguments to - and -- swit-   */
			/* ches (without -/--) are read in before and they should never evalua- */
			/* te here.                                                             */

			PHOEBE_args.KEYWORD_SWITCH = 1;
			declare_string (&PHOEBE_args.KEYWORD, argv[i]);
			}
		}
	}

int variables_init ()
	{
	/* This function initializes all global variables to NULL, so that they may */
	/* be assigned fearlessly by other functions.                               */
	
	/* Strings that aren't constant (ie. the ones that could/would change du-   */
	/* ring program execution *must* be set to NULL first. This is required for */
	/* portable testing of string contents - if it's empty, it's NULL and if    */
	/* it's not, it is initialized.                                             */
	PHOEBE_KEYWORD_FILENAME = NULL;
	PHOEBE_DEFAULTS         = NULL;
	PHOEBE_STARTUP_DIR      = NULL;
	PHOEBE_HOME_DIR         = NULL;
	PHOEBE_CONFIG           = NULL;
	PHOEBE_BASE_DIR         = NULL;
	PHOEBE_SOURCE_DIR       = NULL;
	PHOEBE_DEFAULTS_DIR     = NULL;
	PHOEBE_TEMP_DIR         = NULL;
	PHOEBE_DATA_DIR         = NULL;
	PHOEBE_FF_DIR           = NULL;
	PHOEBE_PLOTTING_PACKAGE = NULL;
	PHOEBE_LD_DIR           = NULL;

	/* Let's declare a global keyword filename string to "Undefined". This is   */
	/* needed for command line keyword filenames' incorporation.                */
	declare_string (&PHOEBE_KEYWORD_FILENAME, "Undefined");

	/* The following are global parameter variables which are identified by the */
	/* widget, qualifier, keyword and description. Since they will be dynamica- */
	/* lly stored by realloc, we need to set them to NULL.                      */
	PHOEBE_parameter.widget      = NULL;
	PHOEBE_parameter.qualifier   = NULL;
	PHOEBE_parameter.keyword     = NULL;
	PHOEBE_parameter.description = NULL;
	PHOEBE_parameter.recno       = 0;

	/* Next we have to do the same for scripter variable:                       */
	PHOEBE_scripter_commands_no = 0;
	PHOEBE_scripter_commands    = NULL;

	/* Much the same as above, we initialize command line argument strings:     */
	PHOEBE_args.KEYWORD          = NULL;
	PHOEBE_args.SCRIPT_NAME      = NULL;

	/* Next are the graph containters:                                          */
	PHOEBE_lc_graph_cache.grno = 0;
	PHOEBE_lc_graph_cache.data = NULL;
	}

int phoebe_init ()
	{
	/* This function initializes all parameters that PHOEBE needs. If every-    */
	/* thing is OK, the return value is 0, otherwise it returns -1.             */

	GtkWidget *warning_window = NULL;
	GtkWidget *readout_widget;

	/* We need one readout string to hold all composite string data before it   */
	/* gets allocated to the final container with declare_string ().            */
	char working_string[255];
	char *working_str = working_string;

	char keyword_string[255];
	char *keyword_str = keyword_string;

	/* This is a read-in string used by getline():                              */
	char *delimeter_str = NULL;

	char *readout_str;

	FILE *config_file;

	/* Assign a current directory (i.e. the directory from which PHOEBE was     */
	/* started to PHOEBE_STARTUP_DIR; this is used for resolving relative       */
	/* pathnames.                                                               */
	getcwd (working_str, 255);
	declare_string (&PHOEBE_STARTUP_DIR, working_str);

	/* Assign a .phoebe configuration path to the PHOEBE_CONFIG variable:       */
	if (getenv ("HOME") == NULL)
		{
		warning_window = create_notice_window (
			"PHOEBE fatal!",
			"Environment entry $HOME missing",
			"PHOEBE cannot find the environment entry HOME. Please set it to your",
			"home directory (e.g. set HOME=/home/your_name) and restart PHOEBE.",
			G_CALLBACK (gtk_main_quit));
		return -1;
		}
	declare_string (&USER_HOME_DIR, getenv ("HOME"));

	/* Although it sounds silly, let's check full permissions of the home di-   */
	/* rectory:                                                                 */
	if (!filename_has_full_permissions (USER_HOME_DIR))
		{
		warning_window = create_notice_window (
			"PHOEBE fatal!",
			"Home directory ($HOME environment) protected",
			"Your home directory is protected and PHOEBE cannot write to it.",
			"Please change permission bits (e.g. 'chmod 755 $HOME' or alike).",
			G_CALLBACK (gtk_main_quit));
		return -1;
		}

	/* Everything OK, let's initialize PHOEBE environment file:                 */
	sprintf (working_str, "%s/.phoebe2", USER_HOME_DIR);
	declare_string (&PHOEBE_HOME_DIR, working_str);
	sprintf (working_str, "%s/phoebe.config", PHOEBE_HOME_DIR);
	declare_string (&PHOEBE_CONFIG, working_str);

	/* Read out the configuration from the config file. Do not fill in the con- */
	/* figuration window widgets from here, because the widgets don't exist yet.*/
	if (filename_exists (PHOEBE_CONFIG))
		{
		config_file = fopen (PHOEBE_CONFIG, "r");

		while (!feof (config_file))
			{
			/* The following line reads the line from the input file and checks if  */
			/* everything went OK; if en error occured (e.g. EoF reached), it bre-  */
			/* aks the loop.                                                        */

			if (fgets (keyword_str, 254, config_file) == NULL) break;

			if (strstr (keyword_str, "PHOEBE_BASE_DIR") != NULL)
				{
				if (sscanf (keyword_str, "PHOEBE_BASE_DIR %s", working_str) != 1) declare_string (&PHOEBE_BASE_DIR, "");
				else declare_string (&PHOEBE_BASE_DIR, working_str);
				}
			if (strstr (keyword_str, "PHOEBE_SOURCE_DIR") != NULL)
				{
				if (sscanf (keyword_str, "PHOEBE_SOURCE_DIR %s", working_str) != 1) declare_string (&PHOEBE_SOURCE_DIR, "");
				else declare_string (&PHOEBE_SOURCE_DIR, working_str);
				}
			if (strstr (keyword_str, "PHOEBE_DEFAULTS_DIR") != NULL)
				{
				if (sscanf (keyword_str, "PHOEBE_DEFAULTS_DIR %s", working_str) != 1) declare_string (&PHOEBE_DEFAULTS_DIR, "");
				else declare_string (&PHOEBE_DEFAULTS_DIR, working_str);
				}
			if (strstr (keyword_str, "PHOEBE_TEMP_DIR") != NULL)
				{
				if (sscanf (keyword_str, "PHOEBE_TEMP_DIR %s", working_str) != 1) declare_string (&PHOEBE_TEMP_DIR, "");
				else declare_string (&PHOEBE_TEMP_DIR, working_str);
				}
			if (strstr (keyword_str, "PHOEBE_DATA_DIR") != NULL)
				{
				if (sscanf (keyword_str, "PHOEBE_DATA_DIR %s", working_str) != 1) declare_string (&PHOEBE_DATA_DIR, "");
				else declare_string (&PHOEBE_DATA_DIR, working_str);
				}
			if (strstr (keyword_str, "PHOEBE_FF_DIR") != NULL)
				{
				if (sscanf (keyword_str, "PHOEBE_FF_DIR %s", working_str) != 1) declare_string (&PHOEBE_FF_DIR, "");
				else declare_string (&PHOEBE_FF_DIR, working_str);
				}
			if (strstr (keyword_str, "PHOEBE_PLOTTING_PACKAGE") != NULL)
				{
				if (sscanf (keyword_str, "PHOEBE_PLOTTING_PACKAGE %s", working_str) != 1) declare_string (&PHOEBE_PLOTTING_PACKAGE, "");
				else declare_string (&PHOEBE_PLOTTING_PACKAGE, working_str);
				}
			if (strstr (keyword_str, "PHOEBE_LD_SWITCH") != NULL)
				if (sscanf (keyword_str, "PHOEBE_LD_SWITCH %d", &PHOEBE_LD_SWITCH) != 1) PHOEBE_LD_SWITCH = 0;
			if (strstr (keyword_str, "PHOEBE_LD_DIR") != NULL)
				{
				if (sscanf (keyword_str, "PHOEBE_LD_DIR %s", working_str) != 1) declare_string (&PHOEBE_LD_DIR, "");
				else declare_string (&PHOEBE_LD_DIR, working_str);
				}
			if (strstr (keyword_str, "PHOEBE_3D_PLOT_CALLBACK_OPTION") != NULL)
				if (sscanf (keyword_str, "PHOEBE_3D_PLOT_CALLBACK_OPTION %d", &PHOEBE_3D_PLOT_CALLBACK_OPTION) != 1) PHOEBE_3D_PLOT_CALLBACK_OPTION = 0;
			if (strstr (keyword_str, "PHOEBE_CONFIRM_ON_SAVE") != NULL)
				if (sscanf (keyword_str, "PHOEBE_CONFIRM_ON_SAVE %d", &PHOEBE_CONFIRM_ON_SAVE) != 1) PHOEBE_CONFIRM_ON_SAVE = 1;
			if (strstr (keyword_str, "PHOEBE_CONFIRM_ON_QUIT") != NULL)
				if (sscanf (keyword_str, "PHOEBE_CONFIRM_ON_QUIT %d", &PHOEBE_CONFIRM_ON_QUIT) != 1) PHOEBE_CONFIRM_ON_QUIT = 1;
			if (strstr (keyword_str, "PHOEBE_WARN_ON_SYNTHETIC_SCATTER") != NULL)
				if (sscanf (keyword_str, "PHOEBE_WARN_ON_SYNTHETIC_SCATTER %d", &PHOEBE_WARN_ON_SYNTHETIC_SCATTER) != 1) PHOEBE_WARN_ON_SYNTHETIC_SCATTER = 1;
			}
		fclose (config_file);
		}

	if (!filename_exists (PHOEBE_CONFIG))             /* Config file is missing */
		{
		warning_window = create_warning_window (
			PHOEBE_VERSION_NUMBER,
			"WELCOME TO PHOEBE!",
			"Since there is no configuration file, I guess this is the first time you started PHOEBE.",
			"I am about to create a hidden PHOEBE configuration directory in your home directory. Is that OK?",
			G_CALLBACK (on_warning_window_configuration_missing_ok_button_clicked),
			G_CALLBACK (gtk_main_quit));

		/* Until the user decides whether he wants to configure PHOEBE or not, we */
		/* should predefine the variables so that PHOEBE may start. These values  */
		/* suppose that PHOEBE was properly installed by the make install script. */
		declare_string (&PHOEBE_BASE_DIR, "/usr/local/share/phoebe2");
		declare_string (&PHOEBE_SOURCE_DIR, "/usr/local/share/phoebe2/src");
		declare_string (&PHOEBE_DEFAULTS_DIR, "/usr/local/share/phoebe2/defaults");
		declare_string (&PHOEBE_TEMP_DIR, "/tmp");
		declare_string (&PHOEBE_DATA_DIR, PHOEBE_STARTUP_DIR);
		declare_string (&PHOEBE_FF_DIR, "/usr/local/share/phoebe2/fluxfiles");
		declare_string (&PHOEBE_PLOTTING_PACKAGE, "internal");
		PHOEBE_LD_SWITCH = 0;
		declare_string (&PHOEBE_LD_DIR, "");
    PHOEBE_3D_PLOT_CALLBACK_OPTION = 0;
    PHOEBE_CONFIRM_ON_SAVE = 1;
    PHOEBE_CONFIRM_ON_QUIT = 1;
    PHOEBE_WARN_ON_SYNTHETIC_SCATTER = 1;
		}

	/* We now set the pixmaps directories. These directories contain all PHOEBE */
	/* related pixmaps and they should be initialized through a config file. If */
	/* that file is missing, we try to set the pixmaps directories relatively.  */

	/* Pixmaps (empty LC/RV/Chi2 plots, logos, ...):                            */
	sprintf (working_str, "%s/pixmaps/", PHOEBE_BASE_DIR);
  add_pixmap_directory (working_str);

	/* Temporary directory:                                                     */
	add_pixmap_directory (PHOEBE_TEMP_DIR);

	if (!filename_exists (PHOEBE_CONFIG))
		{
		/* This occurs when PHOEBE is started for the first time, so next to al-  */
		/* ready defined pixmaps' directories, we define here a local directory,  */
		/* which is necessary only if PHOEBE is started from the src directory.   */

		/* If by any chance PHOEBE is started directly from the src directory:      */
	  add_pixmap_directory ("../pixmaps/");
		}

	/* Now that everything is prepared, let's create and show a main window:    */
	PHOEBE = create_PHOEBE ();
  gtk_widget_show (PHOEBE);

	/* If warning window is created, be sure it's on top of main PHOEBE window: */
	if (GTK_IS_WINDOW (warning_window))
		{
		g_object_set_data (G_OBJECT (PHOEBE), "warning_window", warning_window);
		gtk_window_present (GTK_WINDOW (warning_window));
		}

	/* File selectors should be created in their designated directories:        */
	chdir (PHOEBE_DATA_DIR);
	PHOEBE_data_file_selector             = create_PHOEBE_data_file_selector ();
	PHOEBE_keywords_file_selector         = create_PHOEBE_keywords_file_selector ();
	PHOEBE_logging_service_file_selector  = create_PHOEBE_logging_service_file_selector ();

	/* All other widgets should be created with respect to the base directory:  */
	chdir (PHOEBE_BASE_DIR);
	PHOEBE_about                          = create_PHOEBE_about ();
	PHOEBE_assign_lc_data                 = create_PHOEBE_assign_lc_data ();
	PHOEBE_assign_rv_data                 = create_PHOEBE_assign_rv_data ();
	PHOEBE_assign_sp_data                 = create_PHOEBE_assign_sp_data ();
	PHOEBE_calculate_surface_potentials   = create_PHOEBE_calculate_surface_potentials ();
	PHOEBE_color_menu                     = create_PHOEBE_color_menu ();
	PHOEBE_color_selector                 = create_PHOEBE_color_selector ();
  PHOEBE_configuration                  = create_PHOEBE_configuration ();
  PHOEBE_configuration_file_selector    = create_PHOEBE_configuration_file_selector ();
	PHOEBE_data_sheet                     = create_PHOEBE_data_sheet ();
	PHOEBE_filter_menu                    = create_PHOEBE_filter_menu ();
	PHOEBE_logging_service                = create_PHOEBE_logging_service ();
  PHOEBE_plot_lc                        = create_PHOEBE_plot_lc ();
	PHOEBE_scripter                       = create_PHOEBE_scripter ();
	PHOEBE_statistics                     = create_PHOEBE_statistics ();
/*
	PHOEBE_plot_to_file_selector          = create_PHOEBE_plot_to_file_selector ();
  PHOEBE_calculate_grid                 = create_PHOEBE_calculate_grid ();
	PHOEBE_calculate_grid_edit_parameters = create_PHOEBE_calculate_grid_edit_parameters ();
  PHOEBE_plot_rv                        = create_PHOEBE_plot_rv ();
	PHOEBE_plot_to_file                   = create_PHOEBE_plot_to_file ();
	PHOEBE_ld_interpolation               = create_PHOEBE_ld_interpolation ();
	PHOEBE_dc                             = create_PHOEBE_dc ();
	PHOEBE_dc_correlation_matrix          = create_PHOEBE_dc_correlation_matrix ();
	PHOEBE_fitting_quickbar               = create_PHOEBE_fitting_quickbar ();
	PHOEBE_assign_data_file               = create_PHOEBE_assign_data_file ();
	PHOEBE_assign_rv_data_file            = create_PHOEBE_assign_rv_data_file ();
	PHOEBE_assign_el3                     = create_PHOEBE_assign_el3 ();
	PHOEBE_assign_lc_luminosity           = create_PHOEBE_assign_lc_luminosity ();
	PHOEBE_assign_ld_monochromatic_lc     = create_PHOEBE_assign_ld_monochromatic_lc ();
	PHOEBE_assign_ld_monochromatic_rv     = create_PHOEBE_assign_ld_monochromatic_rv ();
	PHOEBE_assign_primary_spots           = create_PHOEBE_assign_primary_spots ();
	PHOEBE_assign_secondary_spots         = create_PHOEBE_assign_secondary_spots ();
	PHOEBE_assign_weighting               = create_PHOEBE_assign_weighting ();
*/


	/* If this is the first time PHOEBE is started, the Cancel button if the    */
	/* configuration window must exit PHOEBE instead of just hiding a widget.   */
	/* "first_time" qualifier is attached to the PHOEBE_configuration window.   */
	/* The caller function should check this against NULL.                      */
	if (!filename_exists (PHOEBE_CONFIG))
		g_object_set_data (G_OBJECT (PHOEBE_configuration), "first_time", PHOEBE_configuration);

	/* If LD interpolation switch is set on, set the interpolation button sen-  */
	/* sitive:                                                                  */
	readout_widget = lookup_widget (PHOEBE, "ld_tables_button");
	if (PHOEBE_LD_SWITCH == 0) gtk_widget_set_sensitive (readout_widget, FALSE);
	if (PHOEBE_LD_SWITCH == 1) gtk_widget_set_sensitive (readout_widget, TRUE);

	/* Change the version label of the About window to contain information gi-  */
	/* ven in PHOEBE_VERSION_NUMBER and PHOEBE_VERSION_DATE:                    */
	readout_widget = lookup_widget (PHOEBE_about, "about_version_label");
	sprintf (working_str, "Version %s       %s", PHOEBE_VERSION_NUMBER, PHOEBE_VERSION_DATE);
	gtk_label_set_text (GTK_LABEL (readout_widget), working_str);

	/* Create special tags for text formatting in logging service window:       */
	create_all_logging_service_tags ();

	/* Glade cannot deal with greek fonts and other special character. That is  */
	/* why we have to create them by hand.                                      */
	create_and_set_special_labels ();

	/* Glade cannot create GtkListView widgets, so we have to do this by hand.  */
	create_all_info_lists ();

	/* All PHOEBE parameters are referenced globally, so we have to initialize  */
	/* them before they could be used.                                          */
	declare_all_parameters ();

	/* Turn on all scripter commands:                                           */
	declare_all_scripter_commands ();
	add_script_commands_to_scripter_window ();

	/* Finally, we have to set a working directory; this will be the temporary  */
	/* directory, where the user has to have full access to:                    */
	if ( (!filename_is_directory (PHOEBE_TEMP_DIR)) && 
       (!filename_has_full_permissions (PHOEBE_TEMP_DIR)) )
		{
		warning_window = create_notice_window (
			"PHOEBE warning",
			"PHOEBE temporary directory selection issue",
			"The directory you have currently chosen to be temporary (storage for temporary files)",
			"doesn't allow you to write to it. Please change permissions or choose another directory.",
			G_CALLBACK (gtk_widget_destroy));
		}
	else chdir (PHOEBE_TEMP_DIR);

	/* Open the default values:                                                 */
	sprintf (working_str, "%s/default.phoebe", PHOEBE_DEFAULTS_DIR);
	declare_string (&PHOEBE_DEFAULTS, working_str);
	if (!filename_exists (PHOEBE_DEFAULTS))
		{
		warning_window = create_notice_window (
			"PHOEBE notice",
			"Defaults cannot be found",
			"PHOEBE cannot find its defaults file, 'default.phoebe', in the defaults",
			"directory. Please review your settings. Until then, hard-coded settings apply.",
			G_CALLBACK (gtk_widget_destroy));
		}
	else
		handle_open_keyword_file_exceptions (open_keyword_file (PHOEBE_DEFAULTS));

	/* If the plotting package is set to internal, create canvas for the graphs.*/
	if (strcmp (PHOEBE_PLOTTING_PACKAGE, "internal") == 0)
		create_internal_plotter_canvas ();

	/* Finally, set the window title to a generic form:                         */
	gtk_window_set_title (GTK_WINDOW (PHOEBE), "PHOEBE: PHysics Of Eclipsing Binaries");

	/* Report that PHOEBE has started.                                          */
	print_to_status_bar ("PHOEBE Started...");

	/* If the command line contains the keyword file, try to open it:           */
	if (PHOEBE_args.KEYWORD_SWITCH == 1)
		{
		/* We have to make a distinction between absolute and relative filenames: */
		if (PHOEBE_args.KEYWORD[0] == '/')
			sprintf (working_str, "%s", PHOEBE_args.KEYWORD);
		else
			sprintf (working_str, "%s/%s", PHOEBE_STARTUP_DIR, PHOEBE_args.KEYWORD);
		if (filename_exists (working_str))
			{
			handle_open_keyword_file_exceptions (open_keyword_file (working_str));
			print_to_status_bar ("Command-line keyword file opened.");
			}
		else
			warning_window = create_notice_window (
				"PHOEBE warning",
				"PHOEBE keyword file not found",
				"The filename you have supplied for your keyword file cannot be found.",
				"Please open a valid file through File->Open selection.",
				G_CALLBACK (gtk_widget_destroy));
		}

	/* If the script is run non-interactively, run it:                          */
/*
	if (PHOEBE_EXECUTE_SCRIPT == 1)
		{
*/
		/* We have to make a distinction between absolute and relative filenames: */
/*
		if (PHOEBE_EXECUTE_SCRIPT_NAME[0] == '/')
			sprintf (working_str, "%s", PHOEBE_EXECUTE_SCRIPT_NAME);
		else
			sprintf (working_str, "%s/%s", PHOEBE_STARTUP_DIR, PHOEBE_EXECUTE_SCRIPT_NAME);
		if (filename_exists (working_str))
			{
			open_script (working_str);
			on_scripter_execute_script_button_clicked (NULL, NULL);
			}
		else
			phoebe_fatal ("The script you want to execute does not seem to exist.");
		}
*/

	return 0;
	}

void create_and_set_special_labels ()
	{
	/* This function creates and sets special labels - the ones with greek      */
	/* fonts and sub/superscripts.                                              */

	const char Omega_in[] = "";
	char *Omega = g_convert (Omega_in, -1, "UTF-8", "ISO8859-7", NULL, NULL, NULL);

	const char lambda_in[] = "";
	char *lambda = g_convert (lambda_in, -1, "UTF-8", "ISO8859-7", NULL, NULL, NULL);

	const char nu_in[] = "";
	char *nu = g_convert (nu_in, -1, "UTF-8", "ISO8859-7", NULL, NULL, NULL);

	char working_string[255];
	char *working_str = working_string;

	GtkWidget *readout_widget;

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_lambda_label");
	sprintf (working_str, "<big>%s</big> :", lambda);
	gtk_label_set_markup (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_nu_label");
	sprintf (working_str, "<big>%s</big> :", nu);
	gtk_label_set_markup (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_phsv_label");
	sprintf (working_str, "<big>%s</big><sub>1</sub> :", Omega);
	gtk_label_set_markup (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_pcsv_label");
	sprintf (working_str, "<big>%s</big><sub>2</sub> :", Omega);
	gtk_label_set_markup (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_critical_values_l1_label");
	sprintf (working_str, "<big>%s</big> <big>(</big>L<sub>1 </sub><big>)</big> :", Omega);
	gtk_label_set_markup (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_critical_values_l2_label");
	sprintf (working_str, "<big>%s</big> <big>(</big>L<sub>2 </sub><big>)</big> :", Omega);
	gtk_label_set_markup (GTK_LABEL (readout_widget), working_str);
	}

void create_all_logging_service_tags ()
	{
	GtkTextIter iter;
	GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (lookup_widget (PHOEBE_logging_service, "logging_general_information_entry")));

	/* Create all tags and attach them to all logging service tag tables:       */
	create_logging_service_tags (gtk_text_view_get_buffer (GTK_TEXT_VIEW (lookup_widget (PHOEBE_logging_service, "logging_general_information_entry"))));
	create_logging_service_tags (gtk_text_view_get_buffer (GTK_TEXT_VIEW (lookup_widget (PHOEBE_logging_service, "logging_observational_facts_entry"))));
	create_logging_service_tags (gtk_text_view_get_buffer (GTK_TEXT_VIEW (lookup_widget (PHOEBE_logging_service, "logging_analysis_log_entry"))));

	/* Fill in the first buffer:                                                */
  gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "STAR NAME:", -1, "bold",   NULL);
	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\t\t",       -1, "normal", NULL);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\nRA:",      -1, "bold",   NULL);
	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\t\t\t\t\n", -1, "normal", NULL);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "DEC:",       -1, "bold",   NULL);
	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\t\t\t\n",   -1, "normal", NULL);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "HJD0:",      -1, "bold",   NULL);
	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\t\t\t\n",   -1, "normal", NULL);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "PERIOD:",    -1, "bold",   NULL);
	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\t\t\t",     -1, "normal", NULL);

	/* This will place the cursor initially at the end of the first line:       */
	gtk_text_iter_set_offset (&iter, 12);
	gtk_text_buffer_place_cursor (buffer, &iter);
	}

void create_all_info_lists ()
	{
	/* This function simply calls all create_*_info_list functions.             */

	create_data_lc_info_list ();
	create_data_rv_info_list ();
	create_data_sp_info_list ();
	create_flux_levelers_info_list ();
	create_third_light_info_list ();
	create_ld_mono_lc_info_list ();
	create_spots_primary_info_list ();
	create_spots_secondary_info_list ();
	create_ld_mono_rv_info_list ();
	create_plot_lc_data_info_list ();
	create_plot_lc_statistics_info_list ();
	create_lc_statistics_info_list ();
	create_rv_statistics_info_list ();
	create_scripter_info_list ();
	}

void create_data_lc_info_list ()
	{
	/* LC data info list:                                                       */
	/*                                                                          */
	/*   This list contains eight columns:                                      */
	/*   1. "LC"          the consecutive LC number     Type:   STRING          */
	/*   2. "Filename"    experimental data filename    Type:   STRING          */
	/*   3. "Sigma"       data standard deviation       Type:   DOUBLE          */
	/*   4. "Filter"      experimental data filter      Type:   STRING          */
	/*   5. "Column 1"    column 1 (hidden column)      Type:   STRING          */
	/*   6. "Column 2"    column 2 (hidden column)      Type:   STRING          */
	/*   7. "Column 3"    column 3 (hidden column)      Type:   STRING          */
	/*   8. "Weighting"   experimental data weighting   Type:   STRING          */

	GtkWidget *scroller = lookup_widget (PHOEBE, "data_lc_info_list_scroller");
	GtkListStore *info_list;
	GtkWidget *info_list_view;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	info_list = gtk_list_store_new (9, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_FLOAT);
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list", info_list);

	info_list_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (info_list));
	g_signal_connect (G_OBJECT (info_list), "row-changed", G_CALLBACK (on_data_lc_info_list_row_changed), NULL);
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (info_list_view), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_view", info_list_view);
  g_signal_connect (G_OBJECT (info_list_view), "event", G_CALLBACK (on_data_lc_info_double_clicked), NULL);

	renderer = gtk_cell_renderer_text_new ();
	g_object_set (G_OBJECT (renderer), "xalign", 0.5, NULL);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "LC");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (info_list_view), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filename");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 395);
	gtk_tree_view_append_column (GTK_TREE_VIEW (info_list_view), column);

	renderer = gtk_cell_renderer_text_new ();
	g_object_set (G_OBJECT (renderer), "xalign", 0.5, NULL);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Sigma");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 75);
	gtk_tree_view_append_column (GTK_TREE_VIEW (info_list_view), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (info_list_view), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_column_5", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (info_list_view), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_column_6", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (info_list_view), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_column_7", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (info_list_view), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_lc_info_list_column_8", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Weighting");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 7);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 110);
	gtk_tree_view_append_column (GTK_TREE_VIEW (info_list_view), column);

	gtk_container_add (GTK_CONTAINER (scroller), info_list_view);
	gtk_widget_show_all (scroller);
	}

void create_data_rv_info_list ()
	{
	/* RV data info list:                                                       */
	/*                                                                          */
	/*   This list contains four columns:                                       */
	/*   1. "RV"          the consecutive RV number     Type:   STRING          */
	/*   2. "Filename"    experimental data filename    Type:   STRING          */
	/*   3. "Sigma"       data standard deviation       Type:   DOUBLE          */
	/*   4. "Filter"      experimental data filter      Type:   STRING          */
	/*   5. "Column 1"    column 1 (hidden column)      Type:   STRING          */
	/*   6. "Column 2"    column 2 (hidden column)      Type:   STRING          */
	/*   7. "Column 3"    column 3 (hidden column)      Type:   STRING          */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE, "data_rv_info_list_scroller");

	store = gtk_list_store_new (7, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	g_signal_connect (G_OBJECT (store), "row-changed", G_CALLBACK (on_data_rv_info_list_row_changed), NULL);
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_table", table);
  g_signal_connect (G_OBJECT (table), "event", G_CALLBACK (on_data_rv_info_double_clicked), NULL);
	
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "RV");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filename");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 395);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Sigma");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 75);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_column_5", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_column_6", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_rv_info_list_column_7", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_data_sp_info_list ()
	{
	/* Spectra data info list:                                                  */
	/*                                                                          */
	/*   This list contains four columns:                                       */
	/*   1. "SP"          the consecutive SP number     Type:   STRING          */
	/*   2. "Filename"    experimental data filename    Type:   STRING          */
	/*   3. "Sigma"       data standard deviation       Type:   DOUBLE          */
	/*   4. "Filter"      experimental data filter      Type:   STRING          */
	/*   5. "Column 1"    column 1 (hidden column)      Type:   STRING          */
	/*   6. "Column 2"    column 2 (hidden column)      Type:   STRING          */
	/*   7. "Column 3"    column 3 (hidden column)      Type:   STRING          */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE, "data_sp_info_list_scroller");

	store = gtk_list_store_new (7, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_table", table);
  g_signal_connect (G_OBJECT (table), "event", G_CALLBACK (on_data_sp_info_double_clicked), NULL);
	renderer = gtk_cell_renderer_text_new ();

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "SP");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filename");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 395);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Sigma");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 75);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_column_5", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_column_6", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "data_sp_info_list_column_7", column);
	gtk_tree_view_column_set_visible (column, FALSE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_flux_levelers_info_list ()
	{
	/* Flux levelers info list:                                                 */
	/*                                                                          */
	/*   This list contains three columns:                                      */
	/*   1. "Filter"           experimental data filter      Type:   STRING     */
	/*   2. "Primary level"    primary star level (HLA)      Type:   DOUBLE     */
	/*   3. "Secondary level"  secondary star level (CLA)    Type:   DOUBLE     */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE, "surface_flux_levelers_info_list_scroller");

	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	g_object_set_data (G_OBJECT (PHOEBE), "flux_levelers_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "flux_levelers_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "flux_levelers_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "flux_levelers_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Primary level");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "flux_levelers_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Secondary level");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_third_light_info_list ()
	{
	/* Third light info list:                                                   */
	/*                                                                          */
	/*   This list contains three columns:                                      */
	/*   1. "Filter"           experimental data filter      Type:   STRING     */
	/*   2. "3rd light"        third light                   Type:   DOUBLE     */
	/*   3. "Opacity"          opacity function              Type:   DOUBLE     */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE, "surface_third_light_info_list_scroller");

	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	g_object_set_data (G_OBJECT (PHOEBE), "third_light_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "third_light_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "third_light_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "third_light_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "3rd light");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "third_light_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Opacity");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_ld_mono_lc_info_list ()
	{
	/* Monochromatic limb darkening LC info list:                               */
	/*                                                                          */
	/*   This list contains five columns:                                       */
	/*   1. "Filter"      experimental data filter           Type:   STRING     */
	/*   2. "X1"          primary linear coefficient         Type:   DOUBLE     */
	/*   3. "Y1"          primary nonlinear coefficient      Type:   DOUBLE     */
	/*   4. "X2"          secondary linear coefficient       Type:   DOUBLE     */
	/*   5. "Y2"          secondary nonlinear coefficient    Type:   DOUBLE     */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE, "ld_mono_lc_info_list_scroller");

	store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_lc_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_lc_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_lc_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_lc_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "X1");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_lc_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Y1");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 3);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_lc_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "X2");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 4);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_lc_info_list_column_5", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Y2");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 4);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_ld_mono_rv_info_list ()
	{
	/* Monochromatic limb darkening RV info list:                               */
	/*                                                                          */
	/*   This list contains five columns:                                       */
	/*   1. "Filter"      experimental data filter           Type:   STRING     */
	/*   2. "X1"          primary linear coefficient         Type:   DOUBLE     */
	/*   3. "Y1"          primary nonlinear coefficient      Type:   DOUBLE     */
	/*   4. "X2"          secondary linear coefficient       Type:   DOUBLE     */
	/*   5. "Y2"          secondary nonlinear coefficient    Type:   DOUBLE     */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE, "ld_mono_rv_info_list_scroller");

	store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_rv_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_rv_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_rv_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_rv_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "X1");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_rv_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Y1");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 3);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_rv_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "X2");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 4);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "ld_mono_rv_info_list_column_5", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Y2");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 4);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_spots_primary_info_list ()
	{
	/* Spots on primary star info list:                                         */
	/*                                                                          */
	/*   This list contains five columns:                                       */
	/*   1. "Filter"      experimental data filter           Type:   STRING     */
	/*   2. "Latitude"    spot latitude                      Type:   DOUBLE     */
	/*   3. "Longitude"   spot longitude                     Type:   DOUBLE     */
	/*   4. "Radius"      spot radius                        Type:   DOUBLE     */
	/*   5. "Temperature" spot temperature factor            Type:   DOUBLE     */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE, "spots_primary_spots_info_list_scroller");

	store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	g_object_set_data (G_OBJECT (PHOEBE), "spots_primary_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "spots_primary_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_primary_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_primary_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Latitude");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_primary_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Longitude");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 3);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_primary_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Radius");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 4);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_primary_info_list_column_5", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Temperature");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 4);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_spots_secondary_info_list ()
	{
	/* Spots on secondary star info list:                                       */
	/*                                                                          */
	/*   This list contains five columns:                                       */
	/*   1. "Filter"      experimental data filter           Type:   STRING     */
	/*   2. "Latitude"    spot latitude                      Type:   DOUBLE     */
	/*   3. "Longitude"   spot longitude                     Type:   DOUBLE     */
	/*   4. "Radius"      spot radius                        Type:   DOUBLE     */
	/*   5. "Temperature" spot temperature factor            Type:   DOUBLE     */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE, "spots_secondary_spots_info_list_scroller");

	store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	g_object_set_data (G_OBJECT (PHOEBE), "spots_secondary_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "spots_secondary_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_secondary_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_secondary_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Latitude");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_secondary_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Longitude");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 3);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_secondary_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Radius");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 4);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), GTK_TREE_MODEL (store));
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "spots_secondary_info_list_column_5", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Temperature");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 4);
	g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_plot_lc_data_info_list ()
	{
	/* This function creates an info list within the LC plotting window.        */

	GtkWidget *scroller = lookup_widget (PHOEBE_plot_lc, "plot_lc_data_info_list_scroller");
	GtkListStore *datapool;
	GtkWidget *treeview;
  GtkTreeModel *model;
  GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	enum
		{
		PLOT_LC_FILTER,
		PLOT_LC_SYNTHETIC,
		PLOT_LC_EXPERIMENTAL,
		PLOT_LC_SYNCOLOR,
		PLOT_LC_EXPCOLOR,
		PLOT_LC_OFFSET,
		PLOT_LC_EDITABLE,
		PLOT_LC_ALIGNMENT,
		PLOT_LC_COLNO
		};

	datapool = gtk_list_store_new (PLOT_LC_COLNO, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF, GDK_TYPE_PIXBUF, G_TYPE_DOUBLE, G_TYPE_BOOLEAN, G_TYPE_FLOAT);
	model = GTK_TREE_MODEL (datapool);
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_info_list_store", model);

	treeview = gtk_tree_view_new_with_model (model);
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)), GTK_SELECTION_NONE);
	g_object_set_data (G_OBJECT (PHOEBE), "plot_lc_info_list_table", treeview);
	g_object_unref (G_OBJECT (model));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
	g_signal_connect (G_OBJECT (treeview), "button-press-event", G_CALLBACK (on_plot_lc_data_info_list_clicked), NULL);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) PLOT_LC_FILTER);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_add_attribute (column, renderer, "xalign", 7);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_min_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_toggle_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) PLOT_LC_SYNTHETIC);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Synthetic data?");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "active", 1);
	gtk_tree_view_column_add_attribute (column, renderer, "xalign", 7);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
	g_object_set (G_OBJECT (column), "alignment", 0.5, NULL);

  renderer = gtk_cell_renderer_toggle_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) PLOT_LC_EXPERIMENTAL);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Experimental data?");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "active", 2);
	gtk_tree_view_column_add_attribute (column, renderer, "xalign", 7);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

	renderer = gtk_cell_renderer_pixbuf_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) PLOT_LC_SYNCOLOR);
	g_object_set (G_OBJECT (renderer), "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE, NULL);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Synthetic color");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "pixbuf", 3);
	gtk_tree_view_column_add_attribute (column, renderer, "xalign", 7);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

	renderer = gtk_cell_renderer_pixbuf_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) PLOT_LC_EXPCOLOR);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_info_list_column_5", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Experimental color");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "pixbuf", 4);
	gtk_tree_view_column_add_attribute (column, renderer, "xalign", 7);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) PLOT_LC_OFFSET);
  g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (editable_cell_with_double_edited), model);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_info_list_column_6", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Offset");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 5);
	gtk_tree_view_column_add_attribute (column, renderer, "editable", 6);
	gtk_tree_view_column_add_attribute (column, renderer, "xalign", 7);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

	gtk_container_add (GTK_CONTAINER (scroller), treeview);
	gtk_widget_show_all (scroller);
	}

void create_plot_lc_statistics_info_list ()
	{
	/* This function creates an info list within the LC plotting window for     */
	/* statistics entries (chi2 value, sigma value, number of points, ...).     */

	GtkWidget *frame = lookup_widget (PHOEBE_plot_lc, "plot_lc_statistics_frame");
	GtkWidget *scroller;
	GtkListStore *datapool;
	GtkWidget *treeview;
  GtkTreeModel *model;
  GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	enum
		{
		LC_FILTER,
		LC_EXP_POINTS,
		LC_EXP_AVERAGE,
		LC_EXP_MEDIAN,
		LC_SYN_AVERAGE,
		LC_SYN_MEDIAN,
		LC_UNWEIGHTED_SIGMA,
		LC_WEIGHTED_SIGMA,
		LC_COLNO
		};

	scroller = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (frame), scroller);
	gtk_container_set_border_width (GTK_CONTAINER (scroller), 5);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	datapool = gtk_list_store_new (LC_COLNO, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	model = GTK_TREE_MODEL (datapool);
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_store", model);

	treeview = gtk_tree_view_new_with_model (model);
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)), GTK_SELECTION_NONE);
	g_object_set_data (G_OBJECT (PHOEBE), "plot_lc_statistics_info_list_table", treeview);
	g_object_unref (G_OBJECT (model));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) LC_FILTER);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	g_object_set (renderer, "xalign", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", LC_FILTER);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_min_width (column, 90);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) LC_EXP_POINTS);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	g_object_set (renderer, "xalign", 0.5, NULL);
	g_object_set (G_OBJECT (column), "alignment", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "N__exp");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", LC_EXP_POINTS);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) LC_EXP_AVERAGE);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	g_object_set (renderer, "xalign", 0.5, NULL);
	g_object_set (G_OBJECT (column), "alignment", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "<x>__exp");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", LC_EXP_AVERAGE);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) LC_EXP_MEDIAN);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	g_object_set (renderer, "xalign", 0.5, NULL);
	g_object_set (G_OBJECT (column), "alignment", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "M__exp");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", LC_EXP_MEDIAN);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) LC_SYN_AVERAGE);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_column_5", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	g_object_set (G_OBJECT (column), "alignment", 0.5, NULL);
	g_object_set (renderer, "xalign", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "<x>__syn");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", LC_SYN_AVERAGE);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) LC_SYN_MEDIAN);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_column_6", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	g_object_set (G_OBJECT (column), "alignment", 0.5, NULL);
	g_object_set (renderer, "xalign", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "M__syn");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", LC_SYN_MEDIAN);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) LC_UNWEIGHTED_SIGMA);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_column_7", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	g_object_set (renderer, "xalign", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "Unweighted sigma:");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", LC_UNWEIGHTED_SIGMA);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) LC_WEIGHTED_SIGMA);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_statistics_info_list_column_8", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	g_object_set (renderer, "xalign", 0.5, NULL);
	gtk_tree_view_column_set_title (column, "Weighted sigma:");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", LC_WEIGHTED_SIGMA);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

	gtk_container_add (GTK_CONTAINER (scroller), treeview);
	gtk_widget_show_all (scroller);
	}

void create_lc_statistics_info_list ()
	{
	/* Experimental LC data statistics info list:                               */
	/*                                                                          */
	/*   This list contains five columns:                                       */
	/*   1. "Filter"      experimental data filter           Type:   STRING     */
	/*   2. "Points no."  experimental data points number    Type:   INTEGER    */
	/*   3. "Sum"         the sum of dependent values        Type:   DOUBLE     */
	/*   4. "Average"     the average of dependent values    Type:   DOUBLE     */
	/*   5. "Median"      the median of dependent values     Type:   DOUBLE     */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE_statistics, "statistics_lc_info_list_scroller");

	store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_INT, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_lc_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_lc_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_lc_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_lc_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Points no.");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_lc_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Sum");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 3);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_lc_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Average");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 4);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_lc_info_list_column_5", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Median");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 4);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_rv_statistics_info_list ()
	{
	/* Experimental RV data statistics info list:                               */
	/*                                                                          */
	/*   This list contains five columns:                                       */
	/*   1. "Filter"      experimental data filter           Type:   STRING     */
	/*   2. "Points no."  experimental data points number    Type:   INTEGER    */
	/*   3. "Sum"         the sum of dependent values        Type:   DOUBLE     */
	/*   4. "Average"     the average of dependent values    Type:   DOUBLE     */
	/*   5. "Median"      the median of dependent values     Type:   DOUBLE     */

	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE_statistics, "statistics_rv_info_list_scroller");

	store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_INT, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_rv_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_rv_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_rv_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Filter");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_rv_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Points no.");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_rv_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Sum");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 3);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_rv_info_list_column_4", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Average");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 3);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 4);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE_statistics), "statistics_rv_info_list_column_5", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Median");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 4);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void create_scripter_info_list ()
	{
	GtkListStore *store;
	GtkWidget *table;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scroller = lookup_widget (PHOEBE_scripter, "scripter_info_list_scroller");

	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	g_object_set_data (G_OBJECT (PHOEBE), "scripter_info_list_store", store);
	table = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (table), TRUE);
	g_object_set_data (G_OBJECT (PHOEBE), "scripter_info_list_table", table);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 0);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "scripter_info_list_column_1", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Action");
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 0);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 145);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);


	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 1);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "scripter_info_list_column_2", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Arguments");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 1);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 300);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *) 2);
	column = gtk_tree_view_column_new ();
	g_object_set_data (G_OBJECT (PHOEBE), "scripter_info_list_column_3", column);
	gtk_tree_view_column_set_alignment (column, 0.5);
	gtk_tree_view_column_set_title (column, "Status");
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_add_attribute (column, renderer, "text", 2);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (column, 145);
	gtk_tree_view_append_column (GTK_TREE_VIEW (table), column);

	gtk_container_add (GTK_CONTAINER (scroller), table);
	gtk_widget_show_all (scroller);
	}

void add_empty_record_to_all_lc_dependent_info_lists ()
	{
	add_empty_record_to_lc_info_list ();
	add_empty_record_to_flux_levelers_info_list ();
	add_empty_record_to_third_light_info_list ();
	add_empty_record_to_ld_mono_lc_light_info_list ();
	add_empty_record_to_plot_lc_info_list ();
	add_empty_record_to_plot_lc_statistics_info_list ();
	add_empty_record_to_lc_statistics_info_list ();
	}

void add_empty_record_to_all_rv_dependent_info_lists ()
	{
	add_empty_record_to_rv_info_list ();
	add_empty_record_to_ld_mono_rv_light_info_list ();
	add_empty_record_to_rv_statistics_info_list ();
	}

void add_empty_record_to_all_sp_dependent_info_lists ()
	{
	add_empty_record_to_sp_info_list ();
	}

void add_empty_record_to_lc_info_list ()
	{
	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE, "data_lc_info_list"));
	GtkTreeIter iter;

	char lcno_string[5];
	char *lcno_str = lcno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (lc_no < count)
		{
		for (i = count-1; i >= lc_no; i--)
			{
			sprintf (lcno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, lcno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (lc_no > count)
		for (i = count; i < lc_no; i++)
			{
			sprintf (lcno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, lcno_str, 1, "Undefined", 2, 0.01, 3, "Undefined", 4, "Undefined", 5, "Undefined", 6, "Undefined", 7, "Undefined", 8, 1.0, -1);
			}
	}

void add_empty_record_to_plot_lc_info_list ()
	{
	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE_plot_lc, "plot_lc_info_list_store"));
	GtkTreeIter iter;

	char lcno_string[5];
	char *lcno_str = lcno_string;

	int count = 0;
	int i;

	char working_string[255];
	char *working_str = working_string;

	GdkPixbuf *syncolor;
	GdkPixbuf *expcolor;
	GError *error = NULL;

	sprintf (working_str, "%s/pixmaps/color_red.xpm", PHOEBE_BASE_DIR);
	syncolor = gdk_pixbuf_new_from_file (working_str, &error);
	if (error)
		{
		phoebe_warning ("%s\n", error->message);
		g_error_free (error);
		return;
		}
	sprintf (working_str, "%s/pixmaps/color_blue.xpm", PHOEBE_BASE_DIR);
	expcolor = gdk_pixbuf_new_from_file (working_str, &error);
	if (error)
		{
		phoebe_warning ("%s\n", error->message);
		g_error_free (error);
		return;
		}

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (lc_no < count)
		{
		for (i = count-1; i >= lc_no; i--)
			{
			sprintf (lcno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, lcno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (lc_no > count)
		for (i = count; i < lc_no; i++)
			{
			sprintf (lcno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, "Undefined", 1, FALSE, 2, FALSE, 3, syncolor, 4, expcolor, 5, 0.0, 6, TRUE, 7, 0.5, -1);
			g_object_unref (syncolor); g_object_unref (expcolor);
			}
	}

void add_empty_record_to_flux_levelers_info_list ()
	{
	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE, "flux_levelers_info_list_store"));
	GtkTreeIter iter;

	char lcno_string[5];
	char *lcno_str = lcno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (lc_no < count)
		{
		for (i = count-1; i >= lc_no; i--)
			{
			sprintf (lcno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, lcno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (lc_no > count)
		for (i = count; i < lc_no; i++)
			{
			sprintf (lcno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, "Undefined", 1, 10.0, 2, 10.0, -1);
			}
	}

void add_empty_record_to_third_light_info_list ()
	{
	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE, "third_light_info_list_store"));
	GtkTreeIter iter;

	char lcno_string[5];
	char *lcno_str = lcno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (lc_no < count)
		{
		for (i = count-1; i >= lc_no; i--)
			{
			sprintf (lcno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, lcno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (lc_no > count)
		for (i = count; i < lc_no; i++)
			{
			sprintf (lcno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, "Undefined", 1, 0.0, 2, 0.0, -1);
			}
	}

void add_empty_record_to_ld_mono_lc_light_info_list ()
	{
	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE, "ld_mono_lc_info_list_store"));
	GtkTreeIter iter;

	char lcno_string[5];
	char *lcno_str = lcno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (lc_no < count)
		{
		for (i = count-1; i >= lc_no; i--)
			{
			sprintf (lcno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, lcno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (lc_no > count)
		for (i = count; i < lc_no; i++)
			{
			sprintf (lcno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, "Undefined", 1, 0.0, 2, 0.0, 3, 0.0, 4, 0.0, -1);
			}
	}

void add_empty_record_to_ld_mono_rv_light_info_list ()
	{
	int rv_no = phoebe_get_value_int ("phoebe_rvno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE, "ld_mono_rv_info_list_store"));
	GtkTreeIter iter;

	char rvno_string[5];
	char *rvno_str = rvno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (rv_no < count)
		{
		for (i = count-1; i >= rv_no; i--)
			{
			sprintf (rvno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, rvno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (rv_no > count)
		for (i = count; i < rv_no; i++)
			{
			sprintf (rvno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, "Undefined", 1, 0.0, 2, 0.0, 3, 0.0, 4, 0.0, -1);
			}
	}

void add_empty_record_to_rv_info_list ()
	{
	int rv_no = phoebe_get_value_int ("phoebe_rvno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE, "data_rv_info_list_store"));
	GtkTreeIter iter;

	char rvno_string[5];
	char *rvno_str = rvno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (rv_no < count)
		{
		for (i = count-1; i >= rv_no; i--)
			{
			sprintf (rvno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, rvno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (rv_no > count)
		for (i = count; i < rv_no; i++)
			{
			sprintf (rvno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, rvno_str, 1, "Undefined", 2, 0.01, 3, "Undefined", 4, "Undefined", 5, "Undefined", 6, "Undefined", -1);
			}
	}

void add_empty_record_to_sp_info_list ()
	{
	int sp_no = phoebe_get_value_int ("phoebe_spno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE, "data_sp_info_list_store"));
	GtkTreeIter iter;

	char spno_string[5];
	char *spno_str = spno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (sp_no < count)
		{
		for (i = count-1; i >= sp_no; i--)
			{
			sprintf (spno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, spno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (sp_no > count)
		for (i = count; i < sp_no; i++)
			{
			sprintf (spno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, spno_str, 1, "Undefined", 2, "0.01", 3, "Undefined", 4, "Undefined", 5, "Undefined", 6, "Undefined", -1);
			}
	}

void add_empty_record_to_plot_lc_statistics_info_list ()
	{
	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE_plot_lc, "plot_lc_statistics_info_list_store"));
	GtkTreeIter iter;

	char lcno_string[5];
	char *lcno_str = lcno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (lc_no < count)
		{
		for (i = count-1; i >= lc_no; i--)
			{
			sprintf (lcno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, lcno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (lc_no > count)
		for (i = count; i < lc_no; i++)
			{
			sprintf (lcno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, "Undefined", 1, "N/A", 2, "N/A", 3, "N/A", 4, "N/A", 5, "N/A", 6, "N/A", 7, "N/A", -1);
			}
	}

void add_empty_record_to_lc_statistics_info_list ()
	{
	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE_statistics, "statistics_lc_info_list_store"));
	GtkTreeIter iter;

	char lcno_string[5];
	char *lcno_str = lcno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (lc_no < count)
		{
		for (i = count-1; i >= lc_no; i--)
			{
			sprintf (lcno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, lcno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (lc_no > count)
		for (i = count; i < lc_no; i++)
			{
			sprintf (lcno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, "Undefined", 1, 0, 2, 0.0, 3, 0.0, 4, 0.0, -1);
			}
	}

void add_empty_record_to_rv_statistics_info_list ()
	{
	int rv_no = phoebe_get_value_int ("phoebe_rvno_value");
	GtkListStore *store = GTK_LIST_STORE (lookup_widget (PHOEBE_statistics, "statistics_rv_info_list_store"));
	GtkTreeIter iter;

	char lcno_string[5];
	char *lcno_str = lcno_string;

	int count = 0;
	int i;

	/* Count the rows already present in the list:                              */
	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);

	if (rv_no < count)
		{
		for (i = count-1; i >= rv_no; i--)
			{
			sprintf (lcno_str, "%d", i);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, lcno_str);
			gtk_list_store_remove (store, &iter);
			}
		}

	if (rv_no > count)
		for (i = count; i < rv_no; i++)
			{
			sprintf (lcno_str, "%d.", i+1);
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, "Undefined", 1, 0, 2, 0.0, 3, 0.0, 4, 0.0, -1);
			}
	}

int print_to_status_bar (const char *fmt, ...)
	{
	GtkWidget *status_bar = lookup_widget (PHOEBE, "status_bar");

	va_list ap;
	int r;

	char out_string[255];
	char *out = out_string;

	va_start (ap, fmt);
	r = vsprintf (out, fmt, ap);
	va_end (ap);

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), 1, out);

	return r;
	}

void strip_string_tail (char *in)
	{
	in[strlen(in)-1] = '\0';
	return;
	}

void parse_filter_names ()
	{
	/* This function scopes through all defined filter names and checks for sa- */
	/* me filter names for different curves and enumerates them accordingly.    */

	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	int rv_no = phoebe_get_value_int ("phoebe_rvno_value");

	int i, j;
	int length;
	int occurence = 1;

	char temp_string[255];
	char *temp_str = temp_string;

	char *filter_str = NULL;

	/* Let's first strip all strings of '#' enumerators:                        */
	for (i = 0; i < lc_no; i++)
		{
		declare_string (&filter_str, phoebe_get_value_list_string ("phoebe_lc_filter", i));
		if (strchr (filter_str, '#') != NULL)
			{
			length = strlen (filter_str) - strlen (strchr (filter_str, '#')) - 1;
			strncpy (temp_str, filter_str, length);
			temp_str[length] = '\0';
			declare_string (&filter_str, temp_str);
			phoebe_set_value_list_string ("phoebe_lc_filter", i, filter_str);
			}
		}
	for (i = 0; i < rv_no; i++)
		{
		declare_string (&filter_str, phoebe_get_value_list_string ("phoebe_rv_filter", i));
		if (strchr (filter_str, '#') != NULL)
			{
			length = strlen (filter_str) - strlen (strchr (filter_str, '#')) - 1;
			strncpy (temp_str, filter_str, length);
			temp_str[length] = '\0';
			declare_string (&filter_str, temp_str);
			phoebe_set_value_list_string ("phoebe_rv_filter", i, filter_str);
			}
		}

	/* Now that all entries are stripped, we scan through all of them and enu-  */
	/* merate the ones with same filter names:                                  */
	for (i = 0; i < lc_no; i++)
		{
		if (strcmp (phoebe_get_value_list_string ("phoebe_lc_filter", i), "Undefined") == 0) continue;
		for (j = i+1; j < lc_no; j++)
			{
			if (strcmp (phoebe_get_value_list_string ("phoebe_lc_filter", j), "Undefined") == 0) continue;
			if (strcmp (phoebe_get_value_list_string ("phoebe_lc_filter", i), phoebe_get_value_list_string ("phoebe_lc_filter", j)) == 0)
				{
				sprintf (temp_str, "%s #%d", phoebe_get_value_list_string ("phoebe_lc_filter", i), occurence+1);
				phoebe_set_value_list_string ("phoebe_lc_filter", j, temp_str);
				occurence++;
				}
			}
		for (j = 0; j < rv_no; j++)
			{
			if (strcmp (phoebe_get_value_list_string ("phoebe_rv_filter", j), "Undefined") == 0) continue;
			if (strcmp (phoebe_get_value_list_string ("phoebe_lc_filter", i), phoebe_get_value_list_string ("phoebe_rv_filter", j)) == 0)
				{
				sprintf (temp_str, "%s #%d", phoebe_get_value_list_string ("phoebe_lc_filter", i), occurence+1);
				phoebe_set_value_list_string ("phoebe_rv_filter", j, temp_str);
				occurence++;
				}
			}
		if (occurence != 1)
			{
			sprintf (temp_str, "%s #1", phoebe_get_value_list_string ("phoebe_lc_filter", i));
			phoebe_set_value_list_string ("phoebe_lc_filter", i, temp_str);
			}
		occurence = 1;
		}
	for (i = 0; i < rv_no; i++)
		{
		if (strcmp (phoebe_get_value_list_string ("phoebe_rv_filter", i), "Undefined") == 0) continue;
		if (strchr (phoebe_get_value_list_string ("phoebe_rv_filter", i), '#') != NULL) continue;
		for (j = i+1; j < rv_no; j++)
			{
			if (strcmp (phoebe_get_value_list_string ("phoebe_rv_filter", j), "Undefined") == 0) continue;
			if (strcmp (phoebe_get_value_list_string ("phoebe_rv_filter", i), phoebe_get_value_list_string ("phoebe_rv_filter", j)) == 0)
				{
				sprintf (temp_str, "%s #%d", phoebe_get_value_list_string ("phoebe_rv_filter", i), occurence+1);
				phoebe_set_value_list_string ("phoebe_rv_filter", j, temp_str);
				occurence++;
				}
			}
		if (occurence != 1)
			{
			sprintf (temp_str, "%s #1",	phoebe_get_value_list_string ("phoebe_rv_filter", i));
			phoebe_set_value_list_string ("phoebe_rv_filter", i, temp_str);
			}
		occurence = 1;
		}

	free (filter_str);
	}

char *parse_data_line (char *in)
	{
	/* This function takes a string, parses it (cleans it of all comment deli-  */
	/* meters, spaces, tabs and newlines), copies the contents to the newly al- */
	/* located string. The original string isn't modified (passed by value!).   */

	char *value = NULL;

	/* If the line contains the comment delimeter '#', discard everything that  */
	/* follows it (strip the input line):                                       */
	if (strchr (in, '#') != NULL)
		in[strlen(in)-strlen(strchr(in,'#'))] = '\0';

	/* If we have spaces, tabs or newlines in front of the first character in   */
	/* line, remove them by incrementing the pointer by 1:                      */
	while ( (*in ==  ' ') || (*in == '\t') || (*in == '\n') )
		if (strlen (in) != 1) in++;
			else { *in = '\0'; break; }

	/* Let's do the same for the tail of the string:                            */
	while ( (in[strlen(in)-1] ==  ' ') || (in[strlen(in)-1] == '\t') || (in[strlen(in)-1] == '\n') ) strip_string_tail (in);

	if (strlen (in) != 0) declare_string (&value, in);

	/* If we removed everything, the line is empty and we return NULL.          */
	return value;
	}

void put_file_contents_to_text_view (char *filename)
	{
	/* This function reads out the file contents char by char and creates a     */
	/* string, which is converted to UTF8, filled into the buffer and, depen-   */
	/* ding on the caller, put to appropriate TextView widget.                  */

	GtkWidget *caller = g_object_get_data (G_OBJECT (PHOEBE_data_file_selector), "caller");

	char *data = NULL;
	char *utf8_data;
	int i = 0;

	GtkTextBuffer *buffer;

	FILE *in;
	
	if ( (!filename_exists (filename)) || (!filename_is_regular_file (filename)) )
		{
		data = malloc (sizeof (*data));
		data[0] = '\0';
		}
	else
		{
		in = fopen (filename, "r");
		while (!feof (in))
			{
			i++;
			data = realloc (data, i * sizeof (*data));
			data[i-1] = fgetc (in);
			}
		fclose (in);

		/* Close the string:                                                        */
		data[i-1] = '\0';
		}

	/* Transform the data to UTF8 format:                                       */
	utf8_data = g_convert (data, -1, "UTF-8", "ISO8859-15", NULL, NULL, NULL);

	/* This is the part where different widgets for different callers are as-   */
	/* signed:                                                                  */
	if (caller == lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filename_browse_button"))
		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_file_contents_text_view")));
	if (caller == lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filename_browse_button"))
		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_file_contents_text_view")));
	if (caller == lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filename_browse_button"))
		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_file_contents_text_view")));

	/* Fill in the TextView widget from the buffer:                             */
	gtk_text_buffer_set_text (buffer, utf8_data, -1);

	/* Free the buffer:                                                         */
	free (data);
	}

int filename_exists (char *filename)
	{
	/* This function checks whether a file exists.                              */

	struct stat log;
	int check;

	if (filename == NULL) return -1;

	check = stat (filename, &log);
	if (check == 0) return 1;
	else
		if (errno == ENOENT) return 0;
			else return -1;
	}

int filename_has_write_permissions (char *filename)
	{
	/* This function checks whether the supplied filename has write permissions */
	/* for the effective user ID.                                               */

	struct stat log;
	int check;

	if (filename == NULL) return -1;

	check = stat (filename, &log);
	if (check == 0)
		{
		if ( ( (log.st_uid == geteuid ()) && (log.st_mode & S_IWUSR) ) ||
			   ( (log.st_gid == getegid ()) && (log.st_mode & S_IWGRP) ) ||
			   ( (log.st_mode & S_IWOTH) ) )
			return 1; else return 0;
		}
	else return -1;
	}

int filename_has_read_permissions (char *filename)
	{
	/* This function checks whether the supplied filename has read permissions  */
	/* for the effective user ID.                                               */

	struct stat log;
	int check;

	if (filename == NULL) return -1;

	check = stat (filename, &log);
	if (check == 0)
		{
		if ( ( (log.st_uid == geteuid ()) && (log.st_mode & S_IRUSR) ) ||
			   ( (log.st_gid == getegid ()) && (log.st_mode & S_IRGRP) ) ||
			   ( (log.st_mode & S_IROTH) ) )
			return 1; else return 0;
		}
	else return -1;
	}

int filename_has_execute_permissions (char *filename)
	{
	/* This function checks whether the supplied filename has execute permissi- */
	/* ons for the effective user ID.                                           */

	struct stat log;
	int check;

	if (filename == NULL) return -1;

	check = stat (filename, &log);
	if (check == 0)
		{
		if ( ( (log.st_uid == geteuid ()) && (log.st_mode & S_IXUSR) ) ||
			   ( (log.st_gid == getegid ()) && (log.st_mode & S_IXGRP) ) ||
			   ( (log.st_mode & S_IXOTH) ) )
			return 1; else return 0;
		}
	else return -1;
	}

int filename_has_full_permissions (char *filename)
	{
	/* This function checks whether the supplied filename has full permissions  */
	/* for the effective user ID.                                               */

	struct stat log;
	int check;

	if (filename == NULL) return -1;

	check = stat (filename, &log);
	if (check == 0)
		{
		if ( ( (log.st_uid == geteuid ()) && (log.st_mode & S_IRUSR) && (log.st_mode & S_IWUSR) && (log.st_mode & S_IXUSR) ) ||
			   ( (log.st_gid == getegid ()) && (log.st_mode & S_IRGRP) && (log.st_mode & S_IWGRP) && (log.st_mode & S_IXGRP) ) ||
			   ( (log.st_mode & S_IROTH) && (log.st_mode & S_IWOTH) && (log.st_mode & S_IXOTH) ) )
			return 1; else return 0;
		}
	else return -1;
	}

int filename_is_directory (char *filename)
	{
	/* This function checks whether the supplied filename is a directory.       */

	struct stat log;
	int check;

	if (filename == NULL) return -1;

	check = stat (filename, &log);
	if (check == 0)
		{
		if (S_ISDIR (log.st_mode)) return 1; else return 0;
		}
	else return -1;
	}

int filename_is_regular_file (char *filename)
	{
	/* This function checks whether the supplied filename is a regular file.    */

	struct stat log;
	int check;

	if (filename == NULL) return -1;

	check = stat (filename, &log);
	if (check == 0)
		{
		if (S_ISREG (log.st_mode)) return 1; else return 0;
		}
	else return -1;
	}

int declare_string (char **in, char *contents)
	{
	/* This function declares a string with a simple strdup(), but first it     */
	/* checks whether a string was ever initialized. All strings should thus be */
	/* initialized to NULL when declared and filled with a value through this   */
	/* wrapper function.                                                        */

	if (*in != NULL) free (*in);
	*in = strdup (contents);

	return 0;
	}

gint gtk_tree_view_column_get_index (GtkTreeViewColumn *column)
	{
	/* This function returns the column index (its number) that is used when    */
	/* changing the contents of the model.                                      */
	
	GList *cols;
	int counter;

	g_return_val_if_fail (column != NULL, -1);
	g_return_val_if_fail (column->tree_view != NULL, -1);

	cols = gtk_tree_view_get_columns (GTK_TREE_VIEW (column->tree_view));
	counter = g_list_index (cols, (gpointer) column);
	g_list_free	(cols);
	return counter;
	}

void editable_cell_with_double_edited (GtkCellRendererText *cell, const char *path_string, const char *new_text, gpointer data)
	{
	/* This callback changes the contents of a given GtkTreeView cell when the  */
	/* user enters a new value. To use it, it is *essential* to set "column"    */
	/* keyword to the calling cell - this is done when creating columns.        */

	GtkTreeModel *model = GTK_TREE_MODEL (data);
	GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
	GtkTreeIter iter;
	int column = (int) g_object_get_data (G_OBJECT (cell), "column");

	gtk_tree_model_get_iter (model, &iter, path);
	gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, atof (new_text), -1);
	}

void editable_cell_with_string_edited (GtkCellRendererText *cell, const char *path_string, const char *new_text, gpointer data)
	{
	/* This callback changes the contents of a given GtkTreeView cell when the  */
	/* user enters a new string. To use it, it is *essential* to set "column"    */
	/* keyword to the calling cell - this is done when creating columns.        */

	GtkTreeModel *model = GTK_TREE_MODEL (data);
	GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
	GtkTreeIter iter;
	int column = (int) g_object_get_data (G_OBJECT (cell), "column");

	gtk_tree_model_get_iter (model, &iter, path);
	gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, new_text, -1);
	}

void update_lc_statistics_info_list (int row)
	{
	PHOEBE_data data;

	char temp_string[255];
	char *temp_str = temp_string;

	phoebe_debug ("Entering function update_lc_statistics_info_list ()\n");

	if ( ( !filename_exists (phoebe_get_value_list_string ("phoebe_lc_filename", row)) ) ||
	     ( !filename_is_regular_file (phoebe_get_value_list_string ("phoebe_lc_filename", row)) ) )
		{
		phoebe_debug ("Filename isn't defined or valid, exiting the function\n");
		return;
		}

	if (strcmp (phoebe_get_value_list_string ("phoebe_lc_column1", row), "Undefined") == 0)
		{
		phoebe_debug ("Experimental data column 1 undefined, exiting\n");
		return;
		}
	if (strcmp (phoebe_get_value_list_string ("phoebe_lc_column2", row), "Undefined") == 0)
		{
		phoebe_debug ("Experimental data column 2 undefined, exiting\n");
		return;
		}
	if (strcmp (phoebe_get_value_list_string ("phoebe_lc_column3", row), "Undefined") == 0)
		{
		phoebe_debug ("Experimental data column 3 undefined, exiting\n");
		return;
		}

	phoebe_debug ("Experimental data filename valid, proceeding to read-in\n");
	initialize_memory_for_data (&data);
	if (
		read_in_experimental_data (phoebe_get_value_list_string ("phoebe_lc_filename", row), &data,
			get_input_independent_variable (phoebe_get_value_list_string ("phoebe_lc_column1", row) ), -1,
			get_input_dependent_variable   (phoebe_get_value_list_string ("phoebe_lc_column2", row) ), -1,
			get_input_weight               (phoebe_get_value_list_string ("phoebe_lc_column3", row) ), -1)
      == -1)
		phoebe_warning ("Experimental data readout failure in update_lc_statistics_info_list!\n");

	phoebe_debug ("Experimental data read in successfully\n");

	phoebe_debug ("Updating PHOEBE_statistics window entries\n");
	phoebe_set_value_list_int    ("phoebe_lc_ptsno_value",   row, data.ptsno);
	phoebe_set_value_list_double ("phoebe_lc_sum_value",     row, calculate_sum (data));
	phoebe_set_value_list_double ("phoebe_lc_average_value", row, calculate_average (data));
	phoebe_set_value_list_double ("phoebe_lc_median_value",  row, calculate_median (data));
	phoebe_debug ("PHOEBE_statistics window entries updated\n");

	phoebe_debug ("Freeing data\n");
	free_allocated_memory_for_data (&data);
	phoebe_debug ("Leaving function update_lc_statistics_info_list ()\n");
	}

void update_rv_statistics_info_list (int row)
	{
	PHOEBE_data data;

	if ( ( !filename_exists (phoebe_get_value_list_string ("phoebe_rv_filename", row)) ) ||
	     ( !filename_is_regular_file (phoebe_get_value_list_string ("phoebe_rv_filename", row)) ) )
		return;

	if (strcmp (phoebe_get_value_list_string ("phoebe_rv_column1", row), "Undefined") == 0)
		return;
	if (strcmp (phoebe_get_value_list_string ("phoebe_rv_column2", row), "Undefined") == 0)
		return;
	if (strcmp (phoebe_get_value_list_string ("phoebe_rv_column3", row), "Undefined") == 0)
		return;

	initialize_memory_for_data (&data);
	read_in_experimental_data (phoebe_get_value_list_string ("phoebe_rv_filename", row), &data,
		get_input_independent_variable (phoebe_get_value_list_string ("phoebe_rv_column1", row) ), -1,
		get_input_dependent_variable   (phoebe_get_value_list_string ("phoebe_rv_column2", row) ), -1,
		get_input_weight               (phoebe_get_value_list_string ("phoebe_rv_column3", row) ), -1);

	phoebe_set_value_list_int    ("phoebe_rv_ptsno_value",   row, data.ptsno);
	phoebe_set_value_list_double ("phoebe_rv_sum_value",     row, calculate_sum (data));
	phoebe_set_value_list_double ("phoebe_rv_average_value", row, calculate_average (data));
	phoebe_set_value_list_double ("phoebe_rv_median_value",  row, calculate_median (data));

	free_allocated_memory_for_data (&data);
	}

void create_internal_plotter_canvas ()
	{
	/* This function creates all canvas, sets their backgrounds and initializes */
	/* them for use with the internal plotter.                                  */

	GtkWidget *lc_canvas;
	GdkColor   color;

	lc_canvas = gtk_databox_new ();
	gtk_container_add (GTK_CONTAINER (lookup_widget (PHOEBE_plot_lc, "plot_lc_image_frame")), lc_canvas);
	gtk_widget_show (lc_canvas);
	g_object_set_data (G_OBJECT (PHOEBE_plot_lc), "plot_lc_canvas", lc_canvas);

	/* Choose a background: for now let it be light grey.                       */
	color.red = 60000; color.green = 60000; color.blue = 60000;
	gtk_databox_set_background_color (GTK_DATABOX (lc_canvas), color);
	}
