#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "phoebe_allocations.h"
#include "phoebe_calculations.h"
#include "phoebe_error_handling.h"
#include "phoebe_global.h"
#include "phoebe_gui_callbacks.h"
#include "phoebe_gui_exception_handling.h"
#include "phoebe_gui.h"
#include "phoebe_gui_accessories.h"
#include "phoebe_gui_support.h"
#include "phoebe_keywords.h"

/* Move me to phoebe_plotting when the transition is over:                    */
#include <gtkdatabox.h>

void on_switches_toggled (GtkToggleButton *togglebutton, gpointer user_data)
	{
	/* This function takes care of shading, which represents availability of    */
	/* some specific widget; it is attached to a toggle-button change. If the   */
	/* toggle button is turned on, the widget will be available.                */

	if (GTK_TOGGLE_BUTTON (user_data)->active == TRUE)
		gtk_widget_set_sensitive (GTK_WIDGET (togglebutton), TRUE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (togglebutton), FALSE);
	}

void on_inverted_switches_toggled (GtkToggleButton *togglebutton, gpointer user_data)
	{
	/* This function takes care of shading, which represents availability of    */
	/* some specific widget; it is attached to a toggle-button change. If the   */
	/* toggle button is turned on, the widget will be unavailable.              */

	if (GTK_TOGGLE_BUTTON (user_data)->active == TRUE)
		gtk_widget_set_sensitive (GTK_WIDGET (togglebutton), FALSE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (togglebutton), TRUE);
	}

void on_parameter_value_changed_update_ties (GtkSpinButton *spinbutton, gpointer user_data)
	{
	double value = gtk_spin_button_get_value (user_data);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton), value);

	return;
	}

void on_data_lc_no_value_changed (GtkSpinButton *spinbutton, gpointer user_data)
	{
	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	int i;

	add_empty_record_to_all_lc_dependent_info_lists ();

	/* The following part initializes the right dimension of the graph cache    */
	/* memory. If phoebe_realloc() is called with size 0, it calls free(), so   */
	/* it isn't necessary to check for validity here. All newly allocated en-   */
	/* tries will be NULLed, so that all data reading functions work without    */
	/* any need of pre-allocation.                                              */

	if (lc_no > PHOEBE_lc_graph_cache.grno)
		{
		PHOEBE_lc_graph_cache.data = phoebe_realloc (PHOEBE_lc_graph_cache.data, lc_no * 2 * sizeof (PHOEBE_graph_data));
		for (i = 2 * PHOEBE_lc_graph_cache.grno; i < 2 * lc_no; i++)
			initialize_memory_for_graph_data (&PHOEBE_lc_graph_cache.data[i]);
		}
	else
		{
		for (i = 2 * lc_no; i > 2 * PHOEBE_lc_graph_cache.grno; i--)
			free_allocated_memory_for_graph_data (&PHOEBE_lc_graph_cache.data[i]);
		PHOEBE_lc_graph_cache.data = phoebe_realloc (PHOEBE_lc_graph_cache.data, lc_no * sizeof (PHOEBE_graph_data));
		}
	PHOEBE_lc_graph_cache.grno = lc_no;
	}

void on_data_rv_no_value_changed (GtkSpinButton *spinbutton, gpointer user_data)
	{
	add_empty_record_to_all_rv_dependent_info_lists ();
	}

void on_data_sp_no_value_changed (GtkSpinButton *spinbutton, gpointer user_data)
	{
	add_empty_record_to_all_sp_dependent_info_lists ();
	}

void on_warning_window_configuration_missing_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function creates an empty directory .phoebe in the user's $HOME di- */
	/* rectory. If the directory doesn't have write priviligies, PHOEBE will    */
	/* exit.                                                                    */

	GtkWidget *warning_window = lookup_widget (PHOEBE, "warning_window");
	GtkWidget *settings = lookup_widget (PHOEBE, "menu_bar_settings_configuration");
	GtkWidget *notice_window;

	if (!filename_has_write_permissions (USER_HOME_DIR))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE user home directory issue",
			"Your home directory appears to be write-protected.",
			"Please change this and run the configuration again.",
			G_CALLBACK (gtk_main_quit));
		return;
		}

	mkdir (PHOEBE_HOME_DIR, 0755);
	gtk_widget_destroy (warning_window);
	on_menu_bar_settings_configuration_selected (GTK_MENU_ITEM (settings), NULL);

	print_to_status_bar ("PHOEBE Configuration directory %s created...", PHOEBE_HOME_DIR);
	}

void on_warning_window_configuration_file_exists_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function saves the configuration to the .phoebe/phoebe.config file. */

	/* The "success" integer is the error indicator: if it is 0, everything is  */
	/* OK, otherwise it is not. Thus only if success == 0, the configuration    */
	/* should be saved.                                                         */

	int success;
	gtk_widget_destroy (lookup_widget (PHOEBE, "warning_window"));
	success = on_configuration_ok_button_clicked (NULL, NULL);
	if (success == 0) configuration_save_file (NULL, NULL);
	}

void on_warning_window_log_file_exists_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function overwrites the existing log file.                          */

	char *filename = NULL;

	declare_string (&filename, (char *) gtk_file_selection_get_filename (GTK_FILE_SELECTION (PHOEBE_logging_service_file_selector)));
	write_log_to_file (filename);
	print_to_status_bar ("User log %s overwritten.", filename);
	gtk_widget_destroy (lookup_widget (PHOEBE, "warning_window"));
	gtk_widget_hide (PHOEBE_logging_service_file_selector);
	}

void on_warning_window_overwrite_keyword_file_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function overwrites the chosen keyword file.                        */

	const char *filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (PHOEBE_keywords_file_selector));
	declare_string (&PHOEBE_KEYWORD_FILENAME, (char *) filename);
	save_keyword_file (PHOEBE_KEYWORD_FILENAME);
	gtk_widget_destroy (lookup_widget (PHOEBE, "warning_window"));
	gtk_widget_hide (PHOEBE_keywords_file_selector);
	print_to_status_bar ("Keyword file overwritten...");
	}

void on_warning_window_save_keyword_file_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function overwrites the chosen keyword file.                        */

	save_keyword_file (PHOEBE_KEYWORD_FILENAME);
	gtk_widget_destroy (lookup_widget (PHOEBE, "warning_window"));
	gtk_widget_hide (PHOEBE_keywords_file_selector);
	print_to_status_bar ("Keyword file overwritten...");
	}

void on_warning_window_restore_defaults_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function discards all changes (if any were made) and restores defa- */
	/* ults.                                                                    */

	GtkWidget *notice_window;

	gtk_widget_destroy (lookup_widget (PHOEBE, "warning_window"));

	if (!filename_exists (PHOEBE_DEFAULTS))
		{
		notice_window = create_notice_window (
			"PHOEBE Notice",
			"PHOEBE Notice: Defaults file cannot be found",
			"The defaults file, \"default.phoebe\", resides in PHOEBE defaults directory.",
			"Please verify the path to the defaults directory in Settings->Configuration menu.",
			G_CALLBACK (gtk_widget_destroy));
		return;
		}

	handle_open_keyword_file_exceptions (open_keyword_file (PHOEBE_DEFAULTS));
	declare_string (&PHOEBE_KEYWORD_FILENAME, "Undefined");
	print_to_status_bar ("Defaults restored.");
	}

int on_configuration_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function takes the values that are currently in the Configuration   */
	/* window and applies them to the current session. The values do not per-   */
	/* tain after restart.                                                      */

	/* The return value is set to int instead of void so that we can control    */
	/* the successfulness of data assignments. If the function returns 0, eve-  */
	/* rything is OK, if it returns -1, some allocation failed.                 */

	GtkWidget *readout_widget;
	GtkWidget *notice_window;

	char working_string[255];
	char *working_str = working_string;

	const char *readout_str;

	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_base_directory");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if (!filename_is_directory ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE base directory selection issue",
			"The filename you have currently chosen to be the base",
			"directory isn't a directory. Please choose a directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if (!filename_exists ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE base directory selection issue",
			"The filename you have currently chosen to be the base",
			"directory doesn't exist. Please choose an existing directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	declare_string (&PHOEBE_BASE_DIR, (char *) readout_str);

	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_source_directory");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if (!filename_is_directory ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE source directory selection issue",
			"The filename you have currently chosen to be the base",
			"directory isn't a directory. Please choose a directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if (!filename_exists ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE source directory selection issue",
			"The filename you have currently chosen to be the source",
			"directory doesn't exist. Please choose an existing directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	declare_string (&PHOEBE_SOURCE_DIR, (char *) readout_str);

	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_defaults_directory");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if (!filename_is_directory ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE defaults directory selection issue",
			"The filename you have currently chosen to be the defaults",
			"directory isn't a directory. Please choose a directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if (!filename_exists ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE defaults directory selection issue",
			"The filename you have currently chosen to be the defaults",
			"directory doesn't exist. Please choose an existing directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if (!filename_has_read_permissions ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE defaults directory selection issue",
			"The directory you have currently chosen to be the defaults directory doesn't",
			"have read priviliges. Please change permissions or choose another directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	declare_string (&PHOEBE_DEFAULTS_DIR, (char *) readout_str);

	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_working_directory");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if (!filename_is_directory ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE temporary directory selection issue",
			"The filename you have currently chosen to be the temporary",
			"directory isn't a directory. Please choose a directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if (!filename_exists ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE temporary directory selection issue",
			"The filename you have currently chosen to be the temporary",
			"directory doesn't exist. Please choose an existing directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if (!filename_has_write_permissions ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE temporary directory selection issue",
			"The directory you have currently chosen to be temporary (storage for temporary files)",
			"doesn't allow you to write to it. Please change permissions or choose another directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	declare_string (&PHOEBE_TEMP_DIR, (char *) readout_str);

	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_data_directory");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if (!filename_is_directory ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE data directory selection issue",
			"The filename you have currently chosen to be the data",
			"directory isn't a directory. Please choose a directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if (!filename_exists ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE data directory selection issue",
			"The filename you have currently chosen to be the data",
			"directory doesn't exist. Please choose an existing directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if (!filename_has_write_permissions ((char *) readout_str))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE data directory selection issue",
			"The directory you have currently chosen to contain data files doesn't have",
			"write priviliges. Please change permissions or choose another directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	declare_string (&PHOEBE_DATA_DIR, (char *) readout_str);

	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_internal_switch");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		declare_string (&PHOEBE_PLOTTING_PACKAGE, "internal");
	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_sm_switch");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		declare_string (&PHOEBE_PLOTTING_PACKAGE, "sm");
	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_gnuplot_switch");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		declare_string (&PHOEBE_PLOTTING_PACKAGE, "gnuplot");
	readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_pgplot_switch");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		declare_string (&PHOEBE_PLOTTING_PACKAGE, "pgplot");

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_ld_switch");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		PHOEBE_LD_SWITCH = 1;
	else
		PHOEBE_LD_SWITCH = 0;

	if (PHOEBE_LD_SWITCH == 0) gtk_widget_set_sensitive (lookup_widget (PHOEBE, "ld_tables_button"), FALSE);
	if (PHOEBE_LD_SWITCH == 1) gtk_widget_set_sensitive (lookup_widget (PHOEBE, "ld_tables_button"), TRUE);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_ld_directory");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if ( (PHOEBE_LD_SWITCH == 1) && (!filename_is_directory ((char *) readout_str)) )
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE limb darkening directory selection issue",
			"The filename you have currently chosen to be the limb darkening",
			"directory isn't a directory. Please choose a directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	if ( (PHOEBE_LD_SWITCH == 1) && (!filename_exists ((char *) readout_str)) )
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE limb darkening directory selection issue",
			"The filename you have currently chosen to be the limb darkening",
			"directory doesn't exist. Please choose an existing directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}
	declare_string (&PHOEBE_LD_DIR, (char *) readout_str);
	sprintf (working_str, "%s/ld_availability.data", readout_str);
	if ( (PHOEBE_LD_SWITCH == 1) && (!filename_exists (working_str)) )
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE Limb Darkening directory selection issue",
			"The directory you have currently chosen to contain Van Hamme's LD tables doesn't contain the table",
			"'ld_availability.data', which is necessary. Please check permissions or choose another directory.",
			G_CALLBACK (gtk_widget_destroy));
		return -1;
		}

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_3d_plot_callback_option");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		PHOEBE_3D_PLOT_CALLBACK_OPTION = 1;
	else
		PHOEBE_3D_PLOT_CALLBACK_OPTION = 0;

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_confirm_on_save");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		PHOEBE_CONFIRM_ON_SAVE = 1;
	else
		PHOEBE_CONFIRM_ON_SAVE = 0;

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_confirm_on_quit");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		PHOEBE_CONFIRM_ON_QUIT = 1;
	else
		PHOEBE_CONFIRM_ON_QUIT = 0;

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_warn_on_synthetic_scatter");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active == TRUE)
		PHOEBE_WARN_ON_SYNTHETIC_SCATTER = 1;
	else
		PHOEBE_WARN_ON_SYNTHETIC_SCATTER = 0;

	print_to_status_bar ("Temporary PHOEBE configuration applied. Defaults will be restored after restart.");
	gtk_widget_hide (PHOEBE_configuration);

	return 0;
	}

void on_configuration_save_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function is a Save button callback from the Configuration window.   */
	/* It tests the input one by one and if everything is ok, it applies the    */
	/* read values to the global variables and writes them to the configuration */
	/* file.                                                                    */

	GtkWidget *warning_window;

	int success;

	if (filename_exists (PHOEBE_CONFIG))
		{
		warning_window = create_warning_window
			("PHOEBE configuration file already exists",
			 "Overwrite existing PHOEBE configuration?",
			 "A PHOEBE configuration file phoebe.config already exists in PHOEBE home directory.",
			 "Are you sure you want to overwrite it?",
			 G_CALLBACK (on_warning_window_configuration_file_exists_ok_button_clicked),
			 G_CALLBACK (gtk_widget_destroy));
		g_object_set_data (G_OBJECT (PHOEBE), "warning_window", warning_window);
		}
	else
		{
		/* The "success" integer is the error indicator: if it is 0, everything   */
		/* is OK, otherwise it is not. Thus only if success == 0, the configura-  */
		/* tion should be saved.                                                  */
		success = on_configuration_ok_button_clicked (NULL, NULL);
		if (success == 0) configuration_save_file (NULL, NULL);
		}
	}

void on_configuration_cancel_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function closes the Configuration window. If PHOEBE is started for  */
	/* the first time and the user cancelled the configuration, this function   */
	/* should terminate the current session, otherwise it should just close the */
	/* window. See comments in phoebe_init() for details.                       */

	if (g_object_get_data (G_OBJECT (PHOEBE_configuration), "first_time") != NULL)
		on_menu_bar_file_quit_selected (NULL, NULL);
	else
		gtk_widget_hide (PHOEBE_configuration);
	}

void on_configuration_browse_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function displays the file selector when a particular Browse button */
	/* from the Configuration window is clicked. A name "browse_button" is as-  */
	/* signed to this particular button for later identification.               */

	gtk_widget_show (PHOEBE_configuration_file_selector);
	g_object_set_data (G_OBJECT (PHOEBE_configuration_file_selector), "browse_button", button);
	}

void on_configuration_file_selector_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function reads out the file selector filename contents and puts     */
	/* them into the corresponding widget of the Configuration window.          */

	const char *readout_str;

	GtkWidget *file_name = lookup_widget (GTK_WIDGET (PHOEBE_configuration_file_selector), "browse_button");

	readout_str = gtk_file_selection_get_filename (GTK_FILE_SELECTION (PHOEBE_configuration_file_selector));
	gtk_entry_set_text (GTK_ENTRY (file_name), readout_str);

	gtk_widget_hide (PHOEBE_configuration_file_selector);
	}

void on_configuration_file_selector_cancel_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_configuration_file_selector);
	}

/* *******************        Menu bar callbacks         ******************** */

void on_menu_bar_file_new_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	/* This function, issued either by clicking on the File->New menu entry or  */
	/* by clicking on the Defaults button, restores defaults (the state in      */
	/* which PHOEBE started), which are stored in $DEFAULTS/default.phoebe.     */

	GtkWidget *warning_window;

	warning_window = create_warning_window (
		"PHOEBE notice",
		"Restore defaults?",
		"By restoring PHOEBE to defaults, all changes",
		"will be lost! Are you sure you want to continue?",
		G_CALLBACK (on_warning_window_restore_defaults_ok_button_clicked),
		G_CALLBACK (gtk_widget_destroy));
	g_object_set_data (G_OBJECT (PHOEBE), "warning_window", warning_window);
	}

void on_menu_bar_file_open_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	/* This function shows the file selector window and connects the identifi-  */
	/* cation data to the window. g_object_set_data function will assign a      */
	/* widget to the "caller" id from all routines and the one that called the  */
	/* file selector last will be connected to this id.                         */
	
	GtkWidget *caller = lookup_widget (PHOEBE, "open_button");
	g_object_set_data (G_OBJECT (PHOEBE_keywords_file_selector), "caller", caller);
	gtk_widget_show (PHOEBE_keywords_file_selector);
	}

void on_menu_bar_file_save_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	GtkWidget *warning_window;

	if (strcmp (PHOEBE_KEYWORD_FILENAME, "Undefined") == 0)
		on_menu_bar_file_save_as_selected (NULL, NULL);
	else
		{
		if (PHOEBE_CONFIRM_ON_SAVE == 1)
			{
			warning_window = create_warning_window (
				"Warning",
				"Overwrite keyword file?",
			  "This will overwrite an existing keyword file.",
				"Are you sure you want to proceed?",
				G_CALLBACK (on_warning_window_save_keyword_file_ok_button_clicked),
				G_CALLBACK (gtk_widget_destroy));
			g_object_set_data (G_OBJECT (PHOEBE), "warning_window", warning_window);
			return;
			}
		save_keyword_file (PHOEBE_KEYWORD_FILENAME);
		print_to_status_bar ("Current keyword file saved.");
		}
	}

void on_menu_bar_file_save_as_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	GtkWidget *caller = lookup_widget (PHOEBE, "save_button");
	g_object_set_data (G_OBJECT (PHOEBE_keywords_file_selector), "caller", caller);
	gtk_widget_show (PHOEBE_keywords_file_selector);
	}

void on_menu_bar_file_import_from_dci_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	}

void on_menu_bar_file_export_to_dci_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	}

void on_menu_bar_file_quit_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	free (PHOEBE_parameter.widget);
	free (PHOEBE_parameter.qualifier);
	free (PHOEBE_parameter.keyword);
	free (PHOEBE_parameter.description);

	gtk_main_quit ();
	}

void on_menu_bar_settings_configuration_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	/* This callback will fill in the current values to Configuration window.   */
	/* It is very important that this is done here and not in the beginning,    */
	/* because otherwise temporary changes wouldn't be reflected.               */

	GtkWidget *readout_widget;
	
	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_base_directory");
	gtk_entry_set_text (GTK_ENTRY (readout_widget), PHOEBE_BASE_DIR);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_source_directory");
	gtk_entry_set_text (GTK_ENTRY (readout_widget), PHOEBE_SOURCE_DIR);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_defaults_directory");
	gtk_entry_set_text (GTK_ENTRY (readout_widget), PHOEBE_DEFAULTS_DIR);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_working_directory");
	gtk_entry_set_text (GTK_ENTRY (readout_widget), PHOEBE_TEMP_DIR);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_data_directory");
	gtk_entry_set_text (GTK_ENTRY (readout_widget), PHOEBE_DATA_DIR);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_flux_files_directory");
	gtk_entry_set_text (GTK_ENTRY (readout_widget), PHOEBE_FF_DIR);

	if (strcmp (PHOEBE_PLOTTING_PACKAGE, "internal") == 0) readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_internal_switch");
	if (strcmp (PHOEBE_PLOTTING_PACKAGE, "sm") == 0) readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_sm_switch");
	if (strcmp (PHOEBE_PLOTTING_PACKAGE, "gnuplot") == 0) readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_gnuplot_switch");
	if (strcmp (PHOEBE_PLOTTING_PACKAGE, "pgplot") == 0) readout_widget = lookup_widget (GTK_WIDGET (PHOEBE_configuration), "configuration_pgplot_switch");

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_ld_switch");
	if (PHOEBE_LD_SWITCH == 0) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (readout_widget), FALSE);
	if (PHOEBE_LD_SWITCH == 1) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (readout_widget), TRUE);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_ld_directory");
	gtk_entry_set_text (GTK_ENTRY (readout_widget), PHOEBE_LD_DIR);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_3d_plot_callback_option");
	if (PHOEBE_3D_PLOT_CALLBACK_OPTION == 0) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (readout_widget), FALSE);
	if (PHOEBE_3D_PLOT_CALLBACK_OPTION == 1) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (readout_widget), TRUE);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_confirm_on_save");
	if (PHOEBE_CONFIRM_ON_SAVE == 0) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (readout_widget), FALSE);
	if (PHOEBE_CONFIRM_ON_SAVE == 1) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (readout_widget), TRUE);

	readout_widget = lookup_widget (PHOEBE_configuration, "configuration_confirm_on_quit");
	if (PHOEBE_CONFIRM_ON_QUIT == 0) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (readout_widget), FALSE);
	if (PHOEBE_CONFIRM_ON_QUIT == 1) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (readout_widget), TRUE);
	
	gtk_widget_show (PHOEBE_configuration);
	}

void on_menu_bar_help_about_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	gtk_widget_show (PHOEBE_about);
	print_to_status_bar ("About PHOEBE...");
	}

void on_component_phsv_calculate_button_clicked (GtkButton *button, gpointer user_data)
	{
	on_calculate_surface_potentials_critical_value_changed (NULL, NULL);
	gtk_widget_show (PHOEBE_calculate_surface_potentials);
	}

void on_component_pcsv_calculate_button_clicked (GtkButton *button, gpointer user_data)
	{
	on_calculate_surface_potentials_critical_value_changed (NULL, NULL);
	gtk_widget_show (PHOEBE_calculate_surface_potentials);
	}

void on_calculate_surface_potentials_calculate_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function takes the user-inputed parameters from the Surface poten-  */
	/* tials calculation window and feeds them to the calculation function; the */
	/* return value is then put to the spin button in the calculation window,   */
	/* from where it may be used for the main window via Update button.         */

	GtkWidget *readout_widget;
	int elliptic;
	double D, q, r1, r2, f1, lambda, nu;
	double phsv, pcsv;

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_circular_orbit");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active) elliptic = 0; else elliptic = 1;

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_da_value");
	D = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_rm_value");
	q = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_r1_value");
	r1 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_r2_value");
	r2 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));

	if (elliptic == 1)
		{
		readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_f1_value");
		f1 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
		readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_lambda_value");
		lambda = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
		readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_nu_value");
		nu = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
		}

	phsv = calculate_phsv_value (elliptic, D, q, r1, f1, lambda, nu);
	pcsv = calculate_pcsv_value (elliptic, D, q, r2, f1, lambda, nu);

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_phsv_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), phsv);
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_pcsv_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), pcsv);
	}

void on_calculate_surface_potentials_update_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function takes the values from the Surface potentials calculation   */
	/* window and puts them to the main PHOEBE window. It doesn't perform any   */
	/* calculations, so if the user enters numbers by hand, they will be copied */
	/* directly to the main window.                                             */

	GtkWidget *src, *dest;

	src  = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_phsv_value");
	dest = lookup_widget (PHOEBE, "component_phsv_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (dest), gtk_spin_button_get_value (GTK_SPIN_BUTTON (src)));

	src  = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_pcsv_value");
	dest = lookup_widget (PHOEBE, "component_pcsv_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (dest), gtk_spin_button_get_value (GTK_SPIN_BUTTON (src)));
	}

void on_calculate_surface_potentials_critical_value_changed (GtkEditable *editable, gpointer user_data)
	{
	GtkWidget *readout_widget;
	double q, F1;
	double L1, L2;

	char L1string[20];
	char *L1str = L1string;
	char L2string[20];
	char *L2str = L2string;
	
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_rm_value");
	q = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_f1_value");
	F1 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));

	calculate_critical_potentials (q, F1, 0, &L1, &L2);
	sprintf (L1str, "%5.5lf", L1); sprintf (L2str, "%5.5lf", L2);

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_critical_values_l1_value");
	gtk_label_set_text (GTK_LABEL (readout_widget), L1str);
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_critical_values_l2_value");
	gtk_label_set_text (GTK_LABEL (readout_widget), L2str);
	}

void on_calculate_surface_potentials_potential_value_changed (GtkEditable *editable, gpointer user_data)
	{
	/* This function takes the user-inputed parameters from the Surface poten-  */
	/* tials calculation window and feeds them to the calculation function; the */
	/* return value is then put to the spin button in the calculation window,   */
	/* from where it may be used for the main window via Update button.         */

	GtkWidget *readout_widget;
	int elliptic;
	double D, q, r1, r2, f1, lambda, nu;
	double phsv, pcsv;

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_circular_orbit");
	if (GTK_TOGGLE_BUTTON (readout_widget)->active) elliptic = 0; else elliptic = 1;

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_da_value");
	D = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_rm_value");
	q = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_r1_value");
	r1 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_r2_value");
	r2 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));

	if (elliptic == 1)
		{
		readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_f1_value");
		f1 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
		readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_lambda_value");
		lambda = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
		readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_nu_value");
		nu = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
		}

	phsv = calculate_phsv_value (elliptic, D, q, r1, f1, lambda, nu);
	pcsv = calculate_pcsv_value (elliptic, D, q, r2, f1, lambda, nu);

	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_phsv_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), phsv);
	readout_widget = lookup_widget (PHOEBE_calculate_surface_potentials, "calculate_surface_potentials_pcsv_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), pcsv);
	}

void on_keywords_file_selector_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function performs the Open/Save/Save As actions on keyword files.   */
	/* The "caller" id is checked for the PHOEBE_keywords_file_selector widget  */
	/* and from there we get the widget that called this file selector. Depen-  */
	/* ding on this different actions are performed.                            */

	const char *filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (PHOEBE_keywords_file_selector));
	GtkWidget *warning_window;
	GtkWidget *caller = g_object_get_data (G_OBJECT (PHOEBE_keywords_file_selector), "caller");

	if (caller == lookup_widget (PHOEBE, "open_button"))
		{
		/* Let's check whether this file exists and is readable by owner:         */
		if (!filename_exists ((char *) filename))
			{
			warning_window = create_notice_window (
				"Warning",
				"Cannot open the keyword file",
			  "The file you have chosen to open doesn't",
				"exist. Please choose another file.",
				G_CALLBACK (gtk_widget_destroy));
			return;
			}
		if (!filename_is_regular_file ((char *) filename))
			{
			warning_window = create_notice_window (
				"Warning",
				"Cannot open the keyword file",
			  "The file you have chosen to open doesn't seem to be a keyword",
				"file (maybe a directory?). Please choose another file.",
				G_CALLBACK (gtk_widget_destroy));
			return;
			}

		handle_open_keyword_file_exceptions (open_keyword_file (filename));
		print_to_status_bar ("Keyword file opened... ");

		gtk_widget_hide (PHOEBE_keywords_file_selector);
		}

	if (caller == lookup_widget (PHOEBE, "save_button"))
		{
		if (!filename_is_regular_file ((char *) filename))
			{
			warning_window = create_notice_window (
				"Warning",
				"Cannot save the keyword file",
			  "The file you have chosen to save isn't a valid",
				"file (maybe a directory?). Please choose another file.",
				G_CALLBACK (gtk_widget_destroy));
			return;
			}
		if (filename_exists ((char *) filename))
			{
			warning_window = create_warning_window (
				"Warning",
				"Keyword file already exists",
			  "Saving to this filename will overwrite the",
				"previous contents. Overwrite it?",
				G_CALLBACK (on_warning_window_overwrite_keyword_file_ok_button_clicked),
				G_CALLBACK (gtk_widget_destroy));
			g_object_set_data (G_OBJECT (PHOEBE), "warning_window", warning_window);
			return;
			}

		save_keyword_file (filename);
		print_to_status_bar ("Keyword file saved... ");

		gtk_widget_hide (PHOEBE_keywords_file_selector);
		}

	/* Let's write this filename to the global variable:                        */
	declare_string (&PHOEBE_KEYWORD_FILENAME, (char *) filename);
	}

void on_keywords_file_selector_cancel_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_keywords_file_selector);
	}

gboolean on_data_lc_info_double_clicked (GtkWidget *widget, GdkEvent *event, gpointer user_data)
	{
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeView *list = GTK_TREE_VIEW (lookup_widget (PHOEBE, "data_lc_info_list_view"));
	GtkTreeSelection *selection = gtk_tree_view_get_selection (list);
	GtkTreePath *path;

	int lc_no = phoebe_get_value_int ("phoebe_lcno_value");
	int index;

	GtkWidget *readout_widget;
	char *readout_str;

	double readout_dbl;

	if ( (event->type == GDK_2BUTTON_PRESS) && (lc_no != 0) )
		{
		/* First, let's read out the current values from the list:                */
		gtk_tree_selection_get_selected (selection, &model, &iter);

		/* Now let's attach the row index that the user double-clicked:           */
		path = gtk_tree_model_get_path (model, &iter);
		index = atoi (gtk_tree_path_to_string (path));
		gtk_tree_path_free (path);
		g_object_set_data (G_OBJECT (PHOEBE_assign_lc_data), "selection", (int *) index);

		gtk_tree_model_get (model, &iter, 1, &readout_str, -1);

		readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filename_entry");
		if (strcmp (readout_str, "Undefined") != 0)
			gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);
		else
			gtk_entry_set_text (GTK_ENTRY (readout_widget), "");

		/* If the filename is invalid, the buffer of the following function will  */
		/* be empty, so the TextView will be cleared.                             */
		g_object_set_data (G_OBJECT (PHOEBE_data_file_selector), "caller", lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filename_browse_button"));
		put_file_contents_to_text_view (readout_str);

		gtk_tree_model_get (model, &iter, 2, &readout_dbl, -1);
		readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_sigma_value");
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), readout_dbl);

		gtk_tree_model_get (model, &iter, 3, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filter_button_label");
		gtk_label_set_text (GTK_LABEL (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 4, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_column_1_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 5, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_column_2_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 6, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_column_3_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 7, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_weighting_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_widget_show (PHOEBE_assign_lc_data);
		}

	return FALSE;
	}

gboolean on_data_rv_info_double_clicked (GtkWidget *widget, GdkEvent *event, gpointer user_data)
	{
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeView *list = GTK_TREE_VIEW (lookup_widget (PHOEBE, "data_rv_info_list_table"));
	GtkTreeSelection *selection = gtk_tree_view_get_selection (list);
	GtkTreePath *path;

	int rv_no = phoebe_get_value_int ("phoebe_rvno_value");

	GtkWidget *readout_widget;
	char *readout_str;

	double readout_dbl;

	int index;

	if ( (event->type == GDK_2BUTTON_PRESS) && (rv_no != 0) )
		{
		/* First, let's read out the current values from the list:                */
		gtk_tree_selection_get_selected (selection, &model, &iter);

		/* Now let's attach the row index that the user double-clicked:           */
		path = gtk_tree_model_get_path (model, &iter);
		index = atoi (gtk_tree_path_to_string (path));
		gtk_tree_path_free (path);
		g_object_set_data (G_OBJECT (PHOEBE_assign_rv_data), "selection", (int *) index);

		gtk_tree_model_get (model, &iter, 1, &readout_str, -1);

		readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filename_entry");
		if (strcmp (readout_str, "Undefined") != 0)
			gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);
		else
			gtk_entry_set_text (GTK_ENTRY (readout_widget), "");

		/* If the filename is invalid, the buffer of the following function will  */
		/* be empty, so the TextView will be cleared.                             */
		g_object_set_data (G_OBJECT (PHOEBE_data_file_selector), "caller", lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filename_browse_button"));
		put_file_contents_to_text_view (readout_str);

		gtk_tree_model_get (model, &iter, 2, &readout_dbl, -1);
		readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_sigma_value");
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), readout_dbl);

		gtk_tree_model_get (model, &iter, 3, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filter_button_label");
		gtk_label_set_text (GTK_LABEL (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 4, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_column_1_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 5, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_column_2_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 6, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_column_3_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_widget_show (PHOEBE_assign_rv_data);
		}

	return FALSE;
	}

gboolean on_data_sp_info_double_clicked (GtkWidget *widget, GdkEvent *event, gpointer user_data)
	{
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeView *list = GTK_TREE_VIEW (lookup_widget (PHOEBE, "data_sp_info_list_table"));
	GtkTreeSelection *selection = gtk_tree_view_get_selection (list);
	GtkTreePath *path;

	int sp_no = phoebe_get_value_int ("phoebe_spno_value");

	GtkWidget *readout_widget;
	char *readout_str;

	int index;

	if ( (event->type == GDK_2BUTTON_PRESS) && (sp_no != 0) )
		{
		/* First, let's read out the current values from the list:                */
		gtk_tree_selection_get_selected (selection, &model, &iter);

		/* Now let's attach the row index that the user double-clicked:           */
		path = gtk_tree_model_get_path (model, &iter);
		index = atoi (gtk_tree_path_to_string (path));
		gtk_tree_path_free (path);
		g_object_set_data (G_OBJECT (PHOEBE_assign_sp_data), "selection", (int *) index);

		gtk_tree_model_get (model, &iter, 1, &readout_str, -1);

		readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filename_entry");
		if (strcmp (readout_str, "Undefined") != 0)
			gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);
		else
			gtk_entry_set_text (GTK_ENTRY (readout_widget), "");

		/* If the filename is invalid, the buffer of the following function will  */
		/* be empty, so the TextView will be cleared.                             */
		g_object_set_data (G_OBJECT (PHOEBE_data_file_selector), "caller", lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filename_browse_button"));
		put_file_contents_to_text_view (readout_str);

		gtk_tree_model_get (model, &iter, 2, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_sigma_value");
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), atof (readout_str));

		gtk_tree_model_get (model, &iter, 3, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filter_button_label");
		gtk_label_set_text (GTK_LABEL (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 4, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_column_1_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 5, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_column_2_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_tree_model_get (model, &iter, 6, &readout_str, -1);
		readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_column_3_entry");
		gtk_entry_set_text (GTK_ENTRY (readout_widget), readout_str);

		gtk_widget_show (PHOEBE_assign_sp_data);
		}

	return FALSE;
	}

void on_assign_lc_data_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function reads out the contents of the LC assignment window and     */
	/* puts them into the corresponding LC list in the main window. It isn't    */
	/* necessary to check the filename for validity, because it's already been  */
	/* done at file selector allocation.                                        */

	GtkWidget *readout_widget;
	const char *readout_str;
	char *label_str;

	/* Since the assignment window is modular, the user could by mistake or on  */
	/* purpose change the selected row within the main window, which shouldn't  */
	/* affect the assignment. Thus we shouldn't depend on the selection, but    */
	/* rather use internal data set when the list was double-clicked.           */

	int index = (int) g_object_get_data (G_OBJECT (PHOEBE_assign_lc_data), "selection");

	readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filename_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if (strlen (readout_str) != 0)
		phoebe_set_value_list_string ("phoebe_lc_filename", index, readout_str);
	else
		phoebe_set_value_list_string ("phoebe_lc_filename", index, "Undefined");

	readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_sigma_value");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_double ("phoebe_lc_sigma", index, atof (readout_str));

	readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filter_button_label");
	gtk_label_get (GTK_LABEL (readout_widget), &label_str);
	phoebe_set_value_list_string ("phoebe_lc_filter", index, label_str);
	parse_filter_names ();

	readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_column_1_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_lc_column1", index, readout_str);

	readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_column_2_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_lc_column2", index, readout_str);

	readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_column_3_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_lc_column3", index, readout_str);

	readout_widget = lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_weighting_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_lc_weighting", index, readout_str);

	gtk_widget_hide (PHOEBE_assign_lc_data);
	}

void on_assign_lc_data_cancel_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_assign_lc_data);
	}

void on_assign_rv_data_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function reads out the contents of the RV assignment window and     */
	/* puts them into the corresponding RV list in the main window. It isn't    */
	/* necessary to check the filename for validity, because it's already been  */
	/* done at file selector allocation.                                        */

	GtkWidget *readout_widget;
	const char *readout_str;
	char *label_str;

	/* Since the assignment window is modular, the user could by mistake or on  */
	/* purpose change the selected row within the main window, which shouldn't  */
	/* affect the assignment. Thus we shouldn't depend on the selection, but    */
	/* rather use internal data set when the list was double-clicked.           */

	int index = (int) g_object_get_data (G_OBJECT (PHOEBE_assign_rv_data), "selection");

	readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filename_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if (strlen (readout_str) != 0)
		phoebe_set_value_list_string ("phoebe_rv_filename", index, readout_str);
	else
		phoebe_set_value_list_string ("phoebe_rv_filename", index, "Undefined");

	readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_sigma_value");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_double ("phoebe_rv_sigma", index, atof (readout_str));

	readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filter_button_label");
	gtk_label_get (GTK_LABEL (readout_widget), &label_str);
	phoebe_set_value_list_string ("phoebe_rv_filter", index, label_str);
	parse_filter_names ();

	readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_column_1_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_rv_column1", index, readout_str);

	readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_column_2_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_rv_column2", index, readout_str);

	readout_widget = lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_column_3_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_rv_column3", index, readout_str);

	gtk_widget_hide (PHOEBE_assign_rv_data);
	}

void on_assign_rv_data_cancel_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_assign_rv_data);
	}

void on_assign_sp_data_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function reads out the contents of the SP assignment window and     */
	/* puts them into the corresponding SP list in the main window. It isn't    */
	/* necessary to check the filename for validity, because it's already been  */
	/* done at file selector allocation.                                        */

	GtkWidget *readout_widget;
	const char *readout_str;
	char *label_str;

	/* Since the assignment window is modular, the user could by mistake or on  */
	/* purpose change the selected row within the main window, which shouldn't  */
	/* affect the assignment. Thus we shouldn't depend on the selection, but    */
	/* rather use internal data set when the list was double-clicked.           */

	int index = (int) g_object_get_data (G_OBJECT (PHOEBE_assign_sp_data), "selection");

	readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filename_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	if (strlen (readout_str) != 0)
		phoebe_set_value_list_string ("phoebe_sp_filename", index, readout_str);
	else
		phoebe_set_value_list_string ("phoebe_sp_filename", index, "Undefined");

	readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_sigma_value");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_double ("phoebe_sp_sigma", index, atof (readout_str));

	readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filter_button_label");
	gtk_label_get (GTK_LABEL (readout_widget), &label_str);
	phoebe_set_value_list_string ("phoebe_sp_filter", index, label_str);
	parse_filter_names ();

	readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_column_1_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_sp_column1", index, readout_str);

	readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_column_2_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_sp_column2", index, readout_str);

	readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_column_3_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_sp_column3", index, readout_str);

	readout_widget = lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_weighting_entry");
	readout_str = gtk_entry_get_text (GTK_ENTRY (readout_widget));
	phoebe_set_value_list_string ("phoebe_sp_weighting", index, readout_str);

	gtk_widget_hide (PHOEBE_assign_sp_data);
	}

void on_assign_sp_data_cancel_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_assign_sp_data);
	}

void on_data_file_selector_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	/* This function reads out the supplied filename, checks its validity and,  */
	/* if everything's ok, it closes the file selector, puts the filename to    */
	/* the appropriate entry and opens a file preview.                          */

	GtkWidget *caller = g_object_get_data (G_OBJECT (PHOEBE_data_file_selector), "caller");
	GtkTextBuffer *buffer;
	GtkWidget *notice_window;
	char *filename = NULL;

	if (!filename_exists ((char *) gtk_file_selection_get_filename (GTK_FILE_SELECTION (PHOEBE_data_file_selector))))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE file selection issue",
			"The filename you specified doesn't exist.",
			"Please choose another filename.",
			G_CALLBACK (gtk_widget_destroy));
		return;
		}
	if (!filename_is_regular_file ((char *) gtk_file_selection_get_filename (GTK_FILE_SELECTION (PHOEBE_data_file_selector))))
		{
		notice_window = create_notice_window (
			"PHOEBE notice",
			"PHOEBE file selection issue",
			"The filename you specified isn't a regular file (maybe",
			"a directory?). Please choose another filename.",
			G_CALLBACK (gtk_widget_destroy));
		return;
		}

	declare_string (&filename, (char *) gtk_file_selection_get_filename (GTK_FILE_SELECTION (PHOEBE_data_file_selector)));

	/* Let's fill in the TextView:                                              */
	put_file_contents_to_text_view (filename);

	/* Put the filename to the corresponding entry:                             */
	if (caller == lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filename_browse_button"))
		gtk_entry_set_text (GTK_ENTRY (lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filename_entry")), filename);
	if (caller == lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filename_browse_button"))
		gtk_entry_set_text (GTK_ENTRY (lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filename_entry")), filename);
	if (caller == lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filename_browse_button"))
		gtk_entry_set_text (GTK_ENTRY (lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filename_entry")), filename);

	/* Hide the window:                                                         */
	gtk_widget_hide (PHOEBE_data_file_selector);
	}

void on_data_file_selector_cancel_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_data_file_selector);
	}

void on_data_filename_browse_button_clicked (GtkButton *button, gpointer user_data)
	{
	g_object_set_data (G_OBJECT (PHOEBE_data_file_selector), "caller", button);
	gtk_widget_show (PHOEBE_data_file_selector);
	}

void on_filter_menu_specific_filter_activate (GtkMenuItem *menuitem, gpointer user_data)
	{
	#warning "Callback name doesn't conform to SPECS."
	GtkWidget *caller = g_object_get_data (G_OBJECT (PHOEBE_filter_menu), "caller");
	char *readout_str;

	gtk_label_get (GTK_LABEL (GTK_BIN (menuitem)->child), &readout_str);
	if (caller == lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filter_button"))
		gtk_label_set (GTK_LABEL (lookup_widget (PHOEBE_assign_lc_data, "assign_lc_data_filter_button_label")), readout_str);
	if (caller == lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filter_button"))
		gtk_label_set (GTK_LABEL (lookup_widget (PHOEBE_assign_rv_data, "assign_rv_data_filter_button_label")), readout_str);
	if (caller == lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filter_button"))
		gtk_label_set (GTK_LABEL (lookup_widget (PHOEBE_assign_sp_data, "assign_sp_data_filter_button_label")), readout_str);
	}

void on_assign_lc_data_filter_button_clicked (GtkButton *button, gpointer user_data)
	{
	g_object_set_data (G_OBJECT (PHOEBE_filter_menu), "caller", button);
	gtk_menu_popup (GTK_MENU (PHOEBE_filter_menu), NULL, NULL, NULL, NULL, 1, 0);
	}

void on_menu_bar_view_plot_light_curves_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	gtk_widget_show (PHOEBE_plot_lc);
	}

void on_menu_bar_view_plot_rv_curves_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	}

void on_plot_lc_plot_button_clicked (GtkButton *button, gpointer user_data)
	{
	int i;
	int lcno = phoebe_get_value_int ("phoebe_lcno_value");

	char temp_string[255];
	char *temp_str = temp_string;

	int error_switch;
	PHOEBE_data lc;
	GtkWidget *notice_window;

	int state;

	/* The following are gtk_databox plotting widgets and parameters:           */
	GtkWidget *readout_widget;
	GtkWidget *canvas;
	GdkColor   color;
	GtkDataboxValue coordinate1, coordinate2;

	PHOEBE_lc_plot_parameters param;

	/* First, we read in all plot parameters: the following function reads in:  */
	/*                                                                          */
	/*   1) .alias         ...    is data aliased outside [-0.5, 0.5] interval  */
	/*   2) .bin           ...    is data binned                                */
	/*   3) .deredden      ...    should data be dereddened                     */
	/*   4) .synthderedden ...    should synthetic LC data be dereddened        */
	/*   5) .residuals     ...    are residuals plotted instead of overplots    */
	/*   6) .*synthetic    ...    on/off switches for synthetic curves (array)  */
	/*   7) .*experimental ...    on/off switches for experimental curves (ar.) */
	/*   8) .*shift        ...    relative LC plot shifts (array of doubles)    */
	/*   9) .*filter       ...    LC filters (array of doubles)                 */
	/*  10) .indep         ...    requested independent plot variable           */
	/*  11) .dep           ...    requested dependent plot variable             */
	/*  12) .weight        ...    requested error-bar handling (not implement.) */
	/*  13) .phstrt        ...    phase start                                   */
	/*  14) .phend         ...    phase end                                     */
	/*  15) .vertices      ...    number of synthetic LC plot vertices          */

	phoebe_debug ("entering function on_plot_lc_plot_button_clicked ()\n");

	read_in_lc_plot_parameters (&param);

	for (i = 0; i < lcno; i++)
		{
		if (param.synthetic[i] == TRUE)
			{
			initialize_memory_for_data (&lc);
			if (read_in_synthetic_data (&lc, i, param.indep, param.dep, param.weight, param.phstart, param.phend, param.vertices) == -1)
				{
				phoebe_warning ("synthetic data readout failure in on_plot_lc_plot_button_clicked ()\n");
				param.synthetic[i] = FALSE;
				break;
				}

			phoebe_debug ("calculating synthetic data average and updating statistics frame\n");
			sprintf (temp_str, "%3.3lf", calculate_average (lc));
			phoebe_set_value_list_string ("phoebe_plot_lc_asyn_value",  i, temp_str);
			sprintf (temp_str, "%3.3lf", calculate_median (lc));
			phoebe_set_value_list_string ("phoebe_plot_lc_msyn_value",  i, temp_str);
			phoebe_debug ("calculation and updating finished successfully\n");

			if (param.shift[i] != 0.0) shift_data (&lc, param.shift[i]);
			if (param.synthderedden == YES)
				{
				if (param.dep == OUTPUT_TOTAL_FLUX) shift_data (&lc, -calculate_interstellar_reddening_correction (param.filter[i], phoebe_get_value_double ("phoebe_synthredr_value"), phoebe_get_value_double ("phoebe_synthrede_value"), 0));
				if (param.dep == OUTPUT_MAGNITUDE)  shift_data (&lc,  calculate_interstellar_reddening_correction (param.filter[i], phoebe_get_value_double ("phoebe_synthredr_value"), phoebe_get_value_double ("phoebe_synthrede_value"), 1));
				}

			free_allocated_memory_for_graph_data (&PHOEBE_lc_graph_cache.data[2*i]);
			initialize_memory_for_graph_data (&PHOEBE_lc_graph_cache.data[2*i]);
			allocate_memory_for_graph_data (&PHOEBE_lc_graph_cache.data[2*i], lc.ptsno);
			read_in_graph_data (&PHOEBE_lc_graph_cache.data[2*i], lc);

			free_allocated_memory_for_data (&lc);
			}
		else
			{
			phoebe_debug ("clearing out statistics entries for non-plotted curves\n");
			phoebe_set_value_list_string ("phoebe_plot_lc_asyn_value",  i, "N/A");
			phoebe_set_value_list_string ("phoebe_plot_lc_msyn_value",  i, "N/A");
			phoebe_debug ("done clearing out statistics entries for non-plotted curves\n");
			}
		if (param.experimental[i] == TRUE)
			{
			initialize_memory_for_data (&lc);
			state = read_in_experimental_data (
				phoebe_get_value_list_string ("phoebe_lc_filename", i), &lc,
				get_input_independent_variable ((char *) phoebe_get_value_list_string ("phoebe_lc_column1", i)),
				param.indep,
				get_input_dependent_variable ((char *) phoebe_get_value_list_string ("phoebe_lc_column2", i)),
				param.dep,
				get_input_weight ((char *) phoebe_get_value_list_string ("phoebe_lc_column3", i)),
				param.weight);
			if (state == -1)
				{
				/* This means something pretty bad has happened, e.g. a filename is   */
				/* missing or it cannot be opened.                                    */
				phoebe_warning ("experimental data readout failure in on_plot_lc_plot_button_clicked\n");
				notice_window = create_notice_window (
					"PHOEBE Notice",
					"PHOEBE Notice: Experimental data filename cannot be found",
					"There is something wrong with experimental data filename supplied",
					"in the data tab. Please verify the filename path and try again.",
					G_CALLBACK (gtk_widget_destroy));
				param.experimental[i] = FALSE;
				break;
				}
			if (state == 1)
				{
				/* This isn't too bad, it happens when the user wants to plot some-   */
				/* thing that doesn't make sense, e.g. experimental individual star   */
				/* fluxes.                                                            */
				phoebe_warning ("further experimental data plotting suppressed.\n");
				param.experimental[i] = FALSE;
				continue;
				}

			phoebe_debug ("calculating experimental data average and updating statistics frame\n");
			sprintf (temp_str, "%d", lc.ptsno);
			phoebe_set_value_list_string ("phoebe_plot_lc_nexp_value",  i, temp_str);
			sprintf (temp_str, "%3.3lf", calculate_average (lc));
			phoebe_set_value_list_string ("phoebe_plot_lc_aexp_value",  i, temp_str);
			sprintf (temp_str, "%3.3lf", calculate_median (lc));
			phoebe_set_value_list_string ("phoebe_plot_lc_mexp_value",  i, temp_str);
			phoebe_debug ("calculation and updating finished successfully\n");

			if (param.bin      == YES) bin_data (&lc, phoebe_get_value_int ("phoebe_binno_value"));
			if (param.alias    == YES) alias_phase_to_interval (&lc, param.phstart, param.phend);
			if (param.deredden == YES)
				{
				if (param.dep == OUTPUT_TOTAL_FLUX) shift_data (&lc, -calculate_interstellar_reddening_correction (param.filter[i], phoebe_get_value_double ("phoebe_reddeningr_value"), phoebe_get_value_double ("phoebe_reddeninge_value"), 0));
				if (param.dep == OUTPUT_MAGNITUDE)  shift_data (&lc,  calculate_interstellar_reddening_correction (param.filter[i], phoebe_get_value_double ("phoebe_reddeningr_value"), phoebe_get_value_double ("phoebe_reddeninge_value"), 1));
				}
			if (param.shift[i] != 0.0) shift_data (&lc, param.shift[i]);
			free_allocated_memory_for_graph_data (&PHOEBE_lc_graph_cache.data[2*i+1]);
			initialize_memory_for_graph_data (&PHOEBE_lc_graph_cache.data[2*i+1]);
			allocate_memory_for_graph_data (&PHOEBE_lc_graph_cache.data[2*i+1], lc.ptsno);
			read_in_graph_data (&PHOEBE_lc_graph_cache.data[2*i+1], lc);

			free_allocated_memory_for_data (&lc);
			}
		else
			{
			phoebe_debug ("clearing out statistics entries for non-plotted curves\n");
			phoebe_set_value_list_string ("phoebe_plot_lc_nexp_value",  i, "N/A");
			phoebe_set_value_list_string ("phoebe_plot_lc_aexp_value",  i, "N/A");
			phoebe_set_value_list_string ("phoebe_plot_lc_mexp_value",  i, "N/A");
			phoebe_debug ("done clearing out statistics entries for non-plotted curves\n");
			}
		}

	/* ************************************************************************ */

	/* Clean the graph canvas:                                                  */
	canvas = lookup_widget (PHOEBE_plot_lc, "plot_lc_canvas");
	gtk_databox_data_remove_all (GTK_DATABOX (canvas));
	gtk_databox_redraw (GTK_DATABOX (canvas));

	/* Plot experimental data to the canvas:                                    */

	for (i = 0; i < lcno; i++)
		{
		if (param.synthetic[i] == TRUE)
			{
			color.red = 65535; color.green = 0; color.blue = 0;
			gtk_databox_data_add_x_y (GTK_DATABOX (canvas), PHOEBE_lc_graph_cache.data[2*i].ptsno, PHOEBE_lc_graph_cache.data[2*i].indep, PHOEBE_lc_graph_cache.data[2*i].dep, color, GTK_DATABOX_LINES, 3);
			}
		if (param.experimental[i] == TRUE)
			{
			color.red = 0; color.green = 0; color.blue = 65535;
			gtk_databox_data_add_x_y (GTK_DATABOX (canvas), PHOEBE_lc_graph_cache.data[2*i+1].ptsno, PHOEBE_lc_graph_cache.data[2*i+1].indep, PHOEBE_lc_graph_cache.data[2*i+1].dep, color, GTK_DATABOX_POINTS, 3);
			}
		}
	gtk_databox_rescale (GTK_DATABOX (canvas));

	/* If we have magnitudes, we have to flip the graph:                        */
	if (get_output_dependent_variable ((char *) phoebe_get_value_string ("phoebe_lc_dep_variable")) == OUTPUT_MAGNITUDE)
		{
		coordinate1.x = GTK_DATABOX (canvas)->min.x;
		coordinate1.y = GTK_DATABOX (canvas)->max.y;
		coordinate2.x = GTK_DATABOX (canvas)->max.x;
		coordinate2.y = GTK_DATABOX (canvas)->min.y;
		gtk_databox_rescale_with_values (GTK_DATABOX (canvas), coordinate1, coordinate2);
		}

	/* ************************************************************************ */

	/* We have to free params' allocated arrays that tell us what to plot:      */
	free (param.synthetic);
	free (param.experimental);
	free (param.shift);

	phoebe_debug ("leaving function on_plot_lc_plot_button_clicked ()\n");
	}

void on_plot_lc_plot_to_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_plot_lc_clear_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_plot_lc_close_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_plot_lc);
	}

void on_menu_bar_view_data_sheet_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	gtk_widget_show (PHOEBE_data_sheet);
	}

void on_data_lc_info_list_row_changed (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data)
	{
	/* This function is issued when fields within the LC info list are changed. */
	/* Its task is to change the entries within LC dependent info lists.        */

	int i = atoi (gtk_tree_path_to_string (path));
	GtkTreeIter newiter;

	GtkListStore *flux_levelers_list = GTK_LIST_STORE (lookup_widget (PHOEBE, "flux_levelers_info_list_store"));
	GtkListStore *third_light_list = GTK_LIST_STORE (lookup_widget (PHOEBE, "third_light_info_list_store"));
	GtkListStore *ld_mono_lc_list = GTK_LIST_STORE (lookup_widget (PHOEBE, "ld_mono_lc_info_list_store"));
	GtkListStore *plot_lc_list = GTK_LIST_STORE (lookup_widget (PHOEBE_plot_lc, "plot_lc_info_list_store"));
	GtkListStore *plot_lc_statistics_list = GTK_LIST_STORE (lookup_widget (PHOEBE_plot_lc, "plot_lc_statistics_info_list_store"));
	GtkListStore *statistics_list = GTK_LIST_STORE (lookup_widget (PHOEBE_statistics, "statistics_lc_info_list_store"));

	if ( strcmp (phoebe_get_value_list_string ("phoebe_lc_filter", i), "Undefined") != 0 )
		{
		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (plot_lc_list), &newiter, gtk_tree_path_to_string (path));
		gtk_list_store_set (plot_lc_list, &newiter, 0, phoebe_get_value_list_string ("phoebe_lc_filter", i), -1);
		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (plot_lc_statistics_list), &newiter, gtk_tree_path_to_string (path));
		gtk_list_store_set (plot_lc_statistics_list, &newiter, 0, phoebe_get_value_list_string ("phoebe_lc_filter", i), -1);
		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (flux_levelers_list), &newiter, gtk_tree_path_to_string (path));
		gtk_list_store_set (flux_levelers_list, &newiter, 0, phoebe_get_value_list_string ("phoebe_lc_filter", i), -1);
		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (third_light_list), &newiter, gtk_tree_path_to_string (path));
		gtk_list_store_set (third_light_list, &newiter, 0, phoebe_get_value_list_string ("phoebe_lc_filter", i), -1);
		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (ld_mono_lc_list), &newiter, gtk_tree_path_to_string (path));
		gtk_list_store_set (ld_mono_lc_list, &newiter, 0, phoebe_get_value_list_string ("phoebe_lc_filter", i), -1);
		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (statistics_list), &newiter, gtk_tree_path_to_string (path));
		gtk_list_store_set (statistics_list, &newiter, 0, phoebe_get_value_list_string ("phoebe_lc_filter", i), -1);
		}

	update_lc_statistics_info_list (i);
	}

void on_data_rv_info_list_row_changed (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data)
	{
	/* This function is issued when fields within the RV info list are changed. */
	/* Its task is to change the entries within RV dependent info lists.        */

	int i = atoi (gtk_tree_path_to_string (path));
	GtkTreeIter newiter;

	GtkListStore *ld_mono_rv_list = GTK_LIST_STORE (lookup_widget (PHOEBE, "ld_mono_rv_info_list_store"));
	GtkListStore *statistics_list = GTK_LIST_STORE (lookup_widget (PHOEBE_statistics, "statistics_rv_info_list_store"));

	if ( strcmp (phoebe_get_value_list_string ("phoebe_rv_filter", i), "Undefined") != 0 )
		{
		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (ld_mono_rv_list), &newiter, gtk_tree_path_to_string (path));
		gtk_list_store_set (ld_mono_rv_list, &newiter, 0, phoebe_get_value_list_string ("phoebe_rv_filter", i), -1);
		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (statistics_list), &newiter, gtk_tree_path_to_string (path));
		gtk_list_store_set (statistics_list, &newiter, 0, phoebe_get_value_list_string ("phoebe_rv_filter", i), -1);
		}

	update_rv_statistics_info_list (i);
	}

void on_data_statistics_button_clicked (GtkButton *button, gpointer user_data)
	{
	print_to_status_bar ("Experimental data statistics window opened.");
	gtk_widget_show (PHOEBE_statistics);
	}

void on_statistics_close_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_statistics);
	}

void on_system_ephemeris_changed (GtkSpinButton *spinbutton, gpointer user_data)
	{
	/* This function is called whenever either HJD0, PERIOD, DPDT or PSHIFT pa- */
	/* rameter has changed. It then updates the values in the Utilities window, */
	/* which are used in the Phase calculator.                                  */

	GtkWidget *readout_widget;
	char working_string[255];
	char *working_str = working_string;
	
	readout_widget = lookup_widget (PHOEBE, "utilities_phase_calculator_hjd0_value");
	sprintf (working_str, "%8.8lf", phoebe_get_value_double ("phoebe_hjd0_value"));
	gtk_label_set_text (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE, "utilities_phase_calculator_period_value");
	sprintf (working_str, "%8.8lf", phoebe_get_value_double ("phoebe_period_value"));
	gtk_label_set_text (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE, "utilities_phase_calculator_dpdt_value");
	sprintf (working_str, "%8.8lf", phoebe_get_value_double ("phoebe_dpdt_value"));
	gtk_label_set_text (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE, "utilities_phase_calculator_pshift_value");
	sprintf (working_str, "%5.5lf", phoebe_get_value_double ("phoebe_pshift_value"));
	gtk_label_set_text (GTK_LABEL (readout_widget), working_str);

	readout_widget = lookup_widget (PHOEBE, "utilities_phase_calculator_hjd_value");
	on_utilities_phase_calculator_hjd_value_changed (GTK_SPIN_BUTTON (readout_widget), NULL);
	}

void on_utilities_phase_calculator_hjd_value_changed (GtkSpinButton *spinbutton, gpointer user_data)
	{
	/* This function is called whenever the user changed the HJD entry in the   */
	/* Phase calculator part of the Utilities window. It automatically calcula- */
	/* tes the phase and updates the value in the Utilities tab.                */

	GtkWidget *readout_widget;
	double hjd, phase, hjd0, period, dpdt, pshift;

	char working_string[255];
	char *working_str = working_string;

	hjd = gtk_spin_button_get_value_as_float (spinbutton);
	read_in_ephemeris_parameters (&hjd0, &period, &dpdt, &pshift);

	phase = calculate_phase_from_ephemeris (hjd, hjd0, period, dpdt, pshift);

	readout_widget = lookup_widget (PHOEBE, "utilities_phase_calculator_phase_value");
	sprintf (working_str, "%8.8lf", phase);
	gtk_label_set_text (GTK_LABEL (readout_widget), working_str);
	}

gboolean on_plot_lc_data_info_list_clicked (GtkWidget *treeview, GdkEventButton *event, gpointer userdata)
	{
	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
	GtkTreePath *path;
	GtkTreeIter iter;
	GtkTreeViewColumn *column;
	gboolean state;

	phoebe_debug ("click on plot_lc_data_info_list caught.\n");

	if (event->type == GDK_BUTTON_PRESS && event->button == 1)
		{
		phoebe_debug ("event->type shows that left mouse button (no. 1) was clicked.\n");
		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview), event->x, event->y, &path, &column, NULL, NULL);
		if (path != NULL)
			{
			phoebe_debug ("path valid, proceeding to column determination.\n");
			if (gtk_tree_view_column_get_index (column) == 1)
				{
				phoebe_debug ("column no. 1 clicked (synthetic data switch).\n");
				/* The user clicked on synthetic LC data switch.                      */
				gtk_tree_model_get_iter (model, &iter, path);
				gtk_tree_model_get (model, &iter, gtk_tree_view_column_get_index (column), &state, -1);
				phoebe_debug ("path and iter readouts successful.\n");
				if (state == TRUE) state = FALSE; else state = TRUE;
				gtk_list_store_set (GTK_LIST_STORE (model), &iter, gtk_tree_view_column_get_index (column), state, -1);
				phoebe_debug ("switching performed.\n");
				}
			if (gtk_tree_view_column_get_index (column) == 2)
				{
				phoebe_debug ("column no. 2 clicked (experimental data switch).\n");
				/* The user clicked on experimental LC data switch.                   */
				gtk_tree_model_get_iter (model, &iter, path);
				gtk_tree_model_get (model, &iter, gtk_tree_view_column_get_index (column), &state, -1);
				phoebe_debug ("path and iter readouts successful.\n");
				if (state == TRUE) state = FALSE; else state = TRUE;
				gtk_list_store_set (GTK_LIST_STORE (model), &iter, gtk_tree_view_column_get_index (column), state, -1);
				phoebe_debug ("switching performed.\n");
				}
			if (gtk_tree_view_column_get_index (column) == 3)
				{
				phoebe_debug ("column no. 3 clicked (synthetic data color).\n");
				/* The user clicked on synthetic LC color.                            */
				gtk_menu_popup (GTK_MENU (PHOEBE_color_menu), NULL, NULL, NULL, NULL, 1, 0);
				phoebe_debug ("color menu popped up.\n");
				}
			if (gtk_tree_view_column_get_index (column) == 4)
				{
				phoebe_debug ("column no. 4 clicked (experimental data color).\n");
				/* The user clicked on experimental LC color.                         */
				gtk_menu_popup (GTK_MENU (PHOEBE_color_menu), NULL, NULL, NULL, NULL, 1, 0);
				phoebe_debug ("color menu popped up.\n");
				}
			if (gtk_tree_view_column_get_index (column) == 5)
				{
				phoebe_debug ("vertical offset clicked. Editing isn't implemented yet.\n");
				/* The user clicked on the vertical offset.                           */
				}
			phoebe_debug ("done with all necessary actions.\n");
			gtk_tree_path_free (path);
			phoebe_debug ("path memory freed.\n");
			phoebe_debug ("returning TRUE.\n");
			return TRUE;
			}
		}
	phoebe_debug ("path invalid, returning FALSE.\n");
	return FALSE;
	}

void on_utilities_main_sequence_calculator_update_button_clicked (GtkButton *button, gpointer user_data)
	{
	GtkWidget *readout_widget;
	
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_omega1_value");
	phoebe_set_value_double ("phoebe_phsv_value", gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget)));
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_omega2_value");
	phoebe_set_value_double ("phoebe_pcsv_value", gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget)));
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_q_value");
	phoebe_set_value_double ("phoebe_rm_value", gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget)));
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_a_value");
	phoebe_set_value_double ("phoebe_sma_value", gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget)));
	}

void on_utilities_main_sequence_calculator_calculate_button_clicked (GtkButton *button, gpointer user_data)
	{
	double T1, T2, P0;
	double L1, L2, M1, M2, q, a, R1, R2, Omega1, Omega2;

	GtkWidget *readout_widget;

	phoebe_debug ("entering function on_utilities_main_sequence_calculator_calculate_button_clicked ()\n");

	phoebe_debug ("reading out input parameters\n");
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_t1_value");
	T1 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_t2_value");
	T2 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_p0_value");
	P0 = gtk_spin_button_get_value (GTK_SPIN_BUTTON (readout_widget));

	phoebe_debug ("calculating output parameters\n");
	calculate_main_sequence_parameters (T1, T2, P0, &L1, &L2, &M1, &M2, &q, &a, &R1, &R2, &Omega1, &Omega2);

	phoebe_debug ("success. Writing output parameters to the GUI\n");
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_l1_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), L1);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_l2_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), L2);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_m1_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), M1);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_m2_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), M2);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_r1_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), R1);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_r2_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), R2);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_omega1_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), Omega1);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_omega2_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), Omega2);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_q_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), q);
	readout_widget = lookup_widget (PHOEBE, "utilities_main_sequence_calculator_a_value");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (readout_widget), a);
	
	phoebe_debug ("leaving function on_utilities_main_sequence_calculator_calculate_button_clicked ()\n");
	return;
	}

void on_menu_bar_edit_log_window_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	gtk_widget_show (PHOEBE_logging_service);
	}

void on_logging_write_to_file_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_show (PHOEBE_logging_service_file_selector);
	}

void on_logging_service_file_selector_ok_button_clicked (GtkButton *button, gpointer user_data)
	{
	char *filename = NULL;
	GtkWidget *warning_window;

	declare_string (&filename, (char *) gtk_file_selection_get_filename (GTK_FILE_SELECTION (PHOEBE_logging_service_file_selector)));
	if ( (filename_exists (filename)) && (filename_is_directory (filename)) )
		{
		warning_window = create_notice_window
			("Chosen log file is invalid",
			 "Invalid log filename", 
			 "The log file you have chosen to write is",
			 "a directory. Please choose a valid file.",
			 G_CALLBACK (gtk_widget_destroy));
		return;
		}
	if (filename_exists (filename))
		{
		warning_window = create_warning_window
			("Chosen log file already exists",
			 "Overwrite existing log file?",
			 "The log file you have chosen to write already",
			 "exists. Are you sure you want to overwrite it?",
			 G_CALLBACK (on_warning_window_log_file_exists_ok_button_clicked),
			 G_CALLBACK (gtk_widget_destroy));
		g_object_set_data (G_OBJECT (PHOEBE), "warning_window", warning_window);
		return;
		}

	write_log_to_file (filename);
	print_to_status_bar ("User log saved to %s.", filename);
	gtk_widget_hide (PHOEBE_logging_service_file_selector);
	}

void on_logging_service_file_selector_cancel_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_logging_service_file_selector);
	}

void on_data_synthetic_scatter_seed_button_clicked (GtkButton *button, gpointer user_data)
	{
	double seed = calculate_synthetic_scatter_seed ();
	phoebe_set_value_double ("phoebe_scatter_seed", seed);
	}

void on_menu_bar_actions_initiate_scripter_selected (GtkMenuItem *menuitem, gpointer user_data)
	{
	gtk_widget_show (PHOEBE_scripter);
	}

void on_scripter_open_script_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_scripter_save_script_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_scripter_execute_script_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_scripter_close_button_clicked (GtkButton *button, gpointer user_data)
	{
	gtk_widget_hide (PHOEBE_scripter);
	}

void on_scripter_add_action_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_scripter_update_action_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_scripter_delete_action_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_scripter_move_action_up_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_scripter_move_action_down_button_clicked (GtkButton *button, gpointer user_data)
	{
	}

void on_scripter_action_changed (GtkOptionMenu *optionmenu, gpointer user_data)
	{
	char *labels[]  = {"scripter_parameter1_label", "scripter_parameter2_label", "scripter_parameter3_label", "scripter_parameter4_label"};
	char *entries[] = {"scripter_parameter1_entry", "scripter_parameter2_entry", "scripter_parameter3_entry", "scripter_parameter4_entry"};

	int i, j;

	GtkWidget *label;
	char *readout_str;

	phoebe_debug ("on_scripter_action_changed signal emitted\n");

	if (GTK_BIN (optionmenu)->child)
		{
		label = GTK_BIN (optionmenu)->child;
		gtk_label_get (GTK_LABEL (label), &readout_str);
		}

	i = 0;
	while ( (i < PHOEBE_scripter_commands_no) && (strcmp (PHOEBE_scripter_commands[i].description, readout_str) != 0) ) i++;

	for (j = 0; j < PHOEBE_scripter_commands[i].argc; j++)
		{
		label = lookup_widget (PHOEBE_scripter, labels[j]);
		gtk_widget_show (label);
		gtk_label_set (GTK_LABEL (label), PHOEBE_scripter_commands[i].argnames[j]);
		gtk_widget_show (lookup_widget (PHOEBE_scripter, entries[j]));
		}
	for (j = PHOEBE_scripter_commands[i].argc; j < 4; j++)
		{
		gtk_widget_hide (lookup_widget (PHOEBE_scripter, labels[j]));
		gtk_widget_hide (lookup_widget (PHOEBE_scripter, entries[j]));
		}

	phoebe_debug ("on_scripter_action_changed callback function completed\n");
	}
